/***************************************************************************
 *   copyright       : (C) 2012-2021 by Pascal Brachet                     *
 *   https://www.xm1math.net/pdfadd/                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   @license GPL-2.0+ <https://spdx.org/licenses/GPL-2.0+.html>           *
 ***************************************************************************/

#include "arbredialog.h"
#include "aidedialog.h"



#include <QDebug>
#include <QFile>
#include <QDir>
#include <QMessageBox>
#include <QFileDialog>
#include <QFileInfo>
#include <QHeaderView>
#include <QDomProcessingInstruction>
#include <QUrl>
#include <QFontDatabase>
#include <QClipboard>
#include <QRegularExpression>
#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
    #include <QtCore/QTextCodec>
#else
    #include <QtCore5Compat/QTextCodec>
#endif

//#include <QCompleter>

ArbreDialog::ArbreDialog(QWidget *parent)
    :QDialog( parent)
{
ui.setupUi(this);
// QStringList commands=QString("\\overline{},\\frac{}").split(",");
// QCompleter* completer = new QCompleter(commands);
// ui.lineEditItem->setCompleter(completer); 
#if defined(Q_OS_MAC)
setModal(true);
#endif
connect(ui.barreOutils->actionQuitter, SIGNAL(triggered()), this, SLOT(accept()));
connect(ui.barreOutils->actionGenerer, SIGNAL(triggered()), this, SLOT(AfficherCode()));
connect(ui.barreOutils->actionNouveau, SIGNAL(triggered()), this, SLOT(NouvelArbre()));
connect(ui.barreOutils->actionOuvrir, SIGNAL(triggered()), this, SLOT(ChargerArbre()));
connect(ui.barreOutils->actionSauver, SIGNAL(triggered()), this, SLOT(SauverArbre()));
connect(ui.barreOutils->actionAide, SIGNAL(triggered()), this, SLOT(Aide()));
connect(ui.barreOutils->actionExporterPdf, SIGNAL(triggered()), this, SLOT(ExporterPdf()));
connect(ui.barreOutils->actionApercu, SIGNAL(triggered()), this, SLOT(AsyPdfTemp()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(VoirTemp()));

compteur_noeuds=0;

connect(ui.ajouterBrancheButton, SIGNAL(clicked()), this, SLOT(AjouterBranche()));
connect(ui.supprimerBrancheButton, SIGNAL(clicked()), this, SLOT(SupprimerBranche()));
connect(ui.ajouterSousBrancheButton, SIGNAL(clicked()), this, SLOT(AjouterSousBranche()));
connect(ui.lineEditItem, SIGNAL(textEdited(const QString &)),this,SLOT(ActualiserCode(const QString &)));
connect(ui.treeWidget, SIGNAL(currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *)),this,SLOT(ActualiserArbre()));


tempDir=QDir::tempPath();
prefixFile=QDir::homePath();
prefixFile="pdfadd_temp_"+prefixFile.section('/',-1);
prefixFile=QString(QUrl::toPercentEncoding(prefixFile));
prefixFile.remove("%");
asyTempFile=prefixFile+".asy";
pdfTempFile=prefixFile+".pdf";

itemMacros << QString::fromUtf8("\\overline{}");
itemMacros << QString::fromUtf8("\\frac{}{}");
itemMacros << QString::fromUtf8("p_{}()");
ui.comboBoxMacros->addItems(itemMacros);
connect(ui.comboBoxMacros, SIGNAL(activated(int)), this, SLOT(AjouterMacro(int)));

QFont efont;
QFontDatabase fdb;
QStringList xf = fdb.families();
QString deft;
if (xf.contains("Liberation Mono",Qt::CaseInsensitive)) deft="Liberation Mono";
else if (xf.contains("DejaVu Sans Mono",Qt::CaseInsensitive)) deft="DejaVu Sans Mono";
#if defined(Q_OS_MAC)
else if (xf.contains("PT Mono",Qt::CaseInsensitive)) deft="PT Mono";
else if (xf.contains("Andale Mono",Qt::CaseInsensitive)) deft="Andale Mono";
else if (xf.contains("Monaco",Qt::CaseInsensitive)) deft="Monaco";
#endif
#if defined(Q_OS_WIN32)
else if (xf.contains("Courier New",Qt::CaseInsensitive)) deft="Courier New";
#endif
else deft=qApp->font().family();  
efont.setFamily(deft);
efont.setPointSize(qApp->font().pointSize());
ui.textEditCode->setFont(efont);
ui.textEditCode->setWordWrapMode(QTextOption::NoWrap);
ui.lineEditItem->setFont(efont);
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Arbres pondérés : ")+nomFichier);
show();
ui.groupBox3->resize(ui.groupBox3->minimumSizeHint());
//resize(minimumSizeHint());
}

ArbreDialog::~ArbreDialog(){
// if (asy_pdf_proc && asy_pdf_proc->state()==QProcess::Running)
// 	{
// 	asy_pdf_proc->disconnect();
// 	asy_pdf_proc->kill();
// 	delete asy_pdf_proc ;
// 	}
}

void ArbreDialog::closeEvent(QCloseEvent *e)
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Arbres pondérés : "),
					QString::fromUtf8("L'arbre courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverArbre();
		break;
		case 1:
		break;
		case 2:
		default:
		e->ignore();
		return;
		break;
		}
	}
e->accept();
}

void ArbreDialog::accept()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Arbres pondérés : "),
					QString::fromUtf8("L'arbre courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverArbre();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::accept();
}

void ArbreDialog::reject()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Arbres pondérés : "),
					QString::fromUtf8("L'arbre courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverArbre();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::reject();
}

void ArbreDialog::Init()
{
ui.lineEditItem->clear();
ui.textEditCode->clear();
ui.treeWidget->clear();
ui.treeWidget->setColumnCount(1);
ui.treeWidget->header()->hide();
compteur_noeuds=0;
}

void ArbreDialog::AjouterBranche()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
QTreeWidgetItem *newItem = 0;
if (curItem) 
	{
	if (curItem->parent()) newItem = new QTreeWidgetItem(curItem->parent(), curItem);
	else newItem = new QTreeWidgetItem(ui.treeWidget, curItem);
	} 
else newItem = new QTreeWidgetItem(ui.treeWidget);
newItem->setText(0,QString::fromUtf8("branche;noeud"));
compteur_noeuds+=1;
newItem->setData(0,Qt::UserRole,QString("noeud")+QString::number(compteur_noeuds));
ui.treeWidget->setCurrentItem(newItem);
ActualiserArbre();
NettoyerCode();
}

void ArbreDialog::SupprimerBranche()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem) return;
QTreeWidgetItem *nextCurrent = 0;
if (curItem->parent()) 
	{
	int idx = curItem->parent()->indexOfChild(curItem);
	if (idx == curItem->parent()->childCount() - 1) idx--;
	else idx++;
	if (idx < 0) nextCurrent = curItem->parent();
	else nextCurrent = curItem->parent()->child(idx);
	} 
else 
	{
	int idx = ui.treeWidget->indexOfTopLevelItem(curItem);
	if (idx == ui.treeWidget->topLevelItemCount() - 1) idx--;
	else idx++;
	if (idx >= 0) nextCurrent = ui.treeWidget->topLevelItem(idx);
	}
delete curItem;
if (nextCurrent) ui.treeWidget->setCurrentItem(nextCurrent);
ActualiserArbre();
NettoyerCode();
}

void ArbreDialog::AjouterSousBranche()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem) return;
QTreeWidgetItem *newItem = new QTreeWidgetItem(curItem);
newItem->setText(0,QString::fromUtf8("branche;noeud"));
compteur_noeuds+=1;
newItem->setData(0,Qt::UserRole,QString("noeud")+QString::number(compteur_noeuds));
ui.treeWidget->setCurrentItem(newItem);
ActualiserArbre();
NettoyerCode();
}

void ArbreDialog::ActualiserArbre()
{
bool currentItemEnabled = false;
QTreeWidgetItem *current = ui.treeWidget->currentItem();
if (current) 
	{
	currentItemEnabled = true;
	ui.lineEditItem->setText(current->text(0));
	}
ui.lineEditItem->setEnabled(currentItemEnabled);
ui.ajouterSousBrancheButton->setEnabled(currentItemEnabled);
ui.supprimerBrancheButton->setEnabled(currentItemEnabled);
}

void ArbreDialog::ActualiserCode(const QString &text)
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem) return;
curItem->setText(0, text);
NettoyerCode();
}

void ArbreDialog::AjouterMacro(int item)
{
QClipboard *clipboard = QApplication::clipboard();
clipboard->setText(itemMacros.at(item));
ui.lineEditItem->paste();
ui.lineEditItem->setFocus();
}
//*************************************
void ArbreDialog::NettoyerCode()
{
ui.textEditCode->clear();
estModifie=true;
setWindowTitle(QString::fromUtf8("Assistant Arbres pondérés ")+QString::fromUtf8("[modifié]")+" : "+nomFichier);
}

QString ArbreDialog::GenererCode()
{
QString texcode=QString::fromUtf8("// préambule asymptote\n")+preambuleAsy+"\n// code figure\n";
QFile arbreasy(":/documents/arbreasy.txt");
arbreasy.open(QIODevice::ReadOnly);
QString preambule;
QTextStream in(&arbreasy);
while (!in.atEnd()) 
	{
	preambule+= in.readLine()+"\n";
	}
arbreasy.close();
texcode+=preambule;
int nb_branches=ui.treeWidget->topLevelItemCount();
if (nb_branches==0) return texcode;
texcode+="TreeNode root = makeNode(\"\",\"\",true);\n";

if (nb_branches==2)
	{
	texcode+=CodeNoeud(0,ui.treeWidget->topLevelItem(0),true);
	texcode+=CodeNoeud(0,ui.treeWidget->topLevelItem(1),false);
	}
else if (nb_branches==3)
	{
	texcode+=CodeNoeud(0,ui.treeWidget->topLevelItem(0),true);
	texcode+=CodeNoeud(0,ui.treeWidget->topLevelItem(1),true);
	texcode+=CodeNoeud(0,ui.treeWidget->topLevelItem(2),false);
	}
else
	{
	for (int i = 0; i < nb_branches; i++) texcode+=CodeNoeud(0,ui.treeWidget->topLevelItem(i), true);
	}
texcode+="draw(root,(0,0));\n";
return texcode;
}

QString ArbreDialog::CodeNoeud(QTreeWidgetItem *parentItem,QTreeWidgetItem *item, bool haut)
{
QString texcode="";
if (!item) return texcode;
int nb_branches=item->childCount();
QString n="";
QString b="";
QString pid="";
if (item->text(0).contains(";"))
	{
	b="$"+item->text(0).section(";", 0, 0)+"$";
	n="$"+item->text(0).section(";", 1, 1)+"$";
	}
else
	{
	b="Erreur";
	n="Erreur";
	}
if (nb_branches>=0)
	{
	 if (parentItem) pid=parentItem->data(0,Qt::UserRole).toString();
	 else pid="root";
	 if (haut) texcode+="TreeNode "+item->data(0,Qt::UserRole).toString()+"=makeNode("+pid+",\""+n+"\",\""+b+"\",true);\n";
	 else texcode+="TreeNode "+item->data(0,Qt::UserRole).toString()+"=makeNode("+pid+",\""+n+"\",\""+b+"\",false);\n";
	if (nb_branches==2)
		{
		texcode+=CodeNoeud(item,item->child(0),true);
		texcode+=CodeNoeud(item,item->child(1),false);
		}
	else if (nb_branches==3)
		{
		texcode+=CodeNoeud(item,item->child(0),true);
		texcode+=CodeNoeud(item,item->child(1),true);
		texcode+=CodeNoeud(item,item->child(2),false);
		}
	else 
		{
		for (int i = 0; i < nb_branches; i++) texcode+=CodeNoeud(item,item->child(i),true);
		}
	}
return texcode;
}

void ArbreDialog::AfficherCode()
{
QString texcode="";
ui.textEditCode->clear();
texcode+=GenererCode()+"\n";
ui.textEditCode->setPlainText(texcode);
}
//******************************
void ArbreDialog::NouvelArbre()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Arbres pondérés : "),
					QString::fromUtf8("L'arbre courant a été modifié.\nVoulez-vous l'enregistrer avant d'en créer un nouveau?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverArbre();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Arbres pondérés : ")+nomFichier);
}

void ArbreDialog::ItemVersXml(QTreeWidgetItem *item,QDomDocument doc ,QDomElement parent)
{
QDomElement element;
element=doc.createElement("item");
element.setAttribute("code",item->text(0));
parent.appendChild(element);
int nb_branches=item->childCount();
if (nb_branches>0)
	{
	for (int i = 0; i < nb_branches; i++) 
		{
		ItemVersXml(item->child(i),doc,element);
		}
	}
}

void ArbreDialog::SauverArbre()
{
int nb_branches=ui.treeWidget->topLevelItemCount();
if (nb_branches==0) return ;
QDomDocument doc;
QDomProcessingInstruction instr =  doc.createProcessingInstruction("xml","version=\"1.0\" encoding=\"UTF-8\"");
doc.appendChild(instr);
QDomElement root=doc.createElement("Arbre");
doc.appendChild(root);
QString nouveauFichier;
if (nomFichier=="sanstitre" ) 
	{
	nouveauFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Arbre (*.pag)"));
	}
else 
	{
	nouveauFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),nomFichier,QString::fromUtf8("Arbre (*.pag)"));
	}
//nouveauFichier=nomFichier;
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::WriteOnly))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible d'enregistrer le fichier"));
	return;
	}
nomFichier=nouveauFichier;
for (int i = 0; i < nb_branches; i++) ItemVersXml(ui.treeWidget->topLevelItem(i),doc,root);

QTextStream out (&fichier);
doc.save(out,4);
fichier.close();
QFileInfo fi(nomFichier);
dernierRepertoire=fi.absolutePath();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Arbres pondérés : ")+nomFichier);
}

void ArbreDialog::XmlVersItem(QTreeWidgetItem *parentItem, QDomElement element)
{
if (!element.hasAttribute("code")) return;
QTreeWidgetItem *newItem;
if (parentItem) newItem = new QTreeWidgetItem(parentItem);
else newItem = new QTreeWidgetItem(ui.treeWidget);
newItem->setText(0,element.attribute("code"));
compteur_noeuds+=1;
newItem->setData(0,Qt::UserRole,QString("noeud")+QString::number(compteur_noeuds));
ui.treeWidget->setCurrentItem(newItem);
ActualiserArbre();
QDomElement enfant = element.firstChildElement();
while (!enfant.isNull()) 
	{
	XmlVersItem(newItem,enfant);
	enfant = enfant.nextSiblingElement();
	}
}

void ArbreDialog::Ouvrir(QString nouveauFichier)
{
QDomDocument doc;
QFileInfo fi(nouveauFichier);
if (!fi.exists()) return;
dernierRepertoire=fi.absolutePath();
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::ReadOnly)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	return;
	}
if (!doc.setContent(&fichier)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	fichier.close();
	return;
	}
QDomElement root = doc.documentElement();
if (root.tagName() != "Arbre")
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Ce fichier ne correspond pas à un arbre"));
	fichier.close();
	return;
	}
nomFichier=nouveauFichier;
Init();
QDomElement enfant = root.firstChildElement();
while (!enfant.isNull()) 
	{
	XmlVersItem(0,enfant);
	enfant = enfant.nextSiblingElement();
	}
fichier.close();
AfficherCode();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Arbres pondérés : ")+nomFichier);
}

void ArbreDialog::ChargerArbre()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Arbres pondérés : ",
					QString::fromUtf8("L'arbre courant a été modifié.\nVoulez-vous l'enregistrer avant de charger un autre arbre?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverArbre();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QString nouveauFichier = QFileDialog::getOpenFileName(this,QString::fromUtf8("Ouvrir Fichier"),dernierRepertoire,"Arbre (*.pag)");
if (nouveauFichier.isEmpty()) return;
Ouvrir(nouveauFichier);
}

//*******************************
void ArbreDialog::Aide()
{
AideDialog *aideDialog = new AideDialog(this,"arbre");
aideDialog->exec();
}

//*****************************
void ArbreDialog::ExporterPdf()
{
this->disconnect(SIGNAL(asy_pdf_fini()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(ExporterPdfSuite()));
AsyPdfTemp();
}

void ArbreDialog::ExporterPdfSuite()
{
if (!erreurProcessus)
	{
	QString pdfFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Pdf (*.pdf)"));
	if (!pdfFichier.isEmpty())
		{
		QFileInfo fi(pdfFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+pdfTempFile,pdfFichier);
		}
	}
this->disconnect(SIGNAL(asy_pdf_fini()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(VoirTemp()));
}

void ArbreDialog::CopierFichier(QString origine,QString destination)
{
if (destination.isEmpty() || origine.isEmpty()) return;
QFileInfo fi_or(origine);
if (!fi_or.exists()) return;
QFile fichier_or(origine);
QFileInfo fi_dest(destination);
if (fi_dest.exists())
	{
	QFile fichier_dest(destination);
	fichier_dest.remove();
	fichier_or.copy(destination);
	}
else
	{
	fichier_or.copy(destination);
	}
}

void ArbreDialog::AsyPdfTemp()
{
AfficherCode();
if (ui.textEditCode->toPlainText().contains(QRegularExpression("Erreur|erreur")))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Erreurs dans le code généré.\nLa compilation n'est pas lancée."));
	return;
	}
QFile fichier;
fichier.setFileName(tempDir+"/"+asyTempFile);
fichier.open(QIODevice::WriteOnly);
QTextCodec *codec = QTextCodec::codecForName("UTF-8");
fichier.write(codec->fromUnicode(GenererCode()));
fichier.close();
erreurProcessus=false;

asy_pdf_proc = new CmdLauncher( this, tempDir, extraPath );
connect(asy_pdf_proc, SIGNAL(jobdone()),this, SIGNAL(asy_pdf_fini()));
connect(asy_pdf_proc, SIGNAL(joberror()),this, SLOT(ErreurCompilation()));
asy_pdf_proc->launchCommand(asyCommande+" -f pdf -noView "+asyTempFile);

}

void ArbreDialog::ErreurCompilation()
{
erreurProcessus=true;
}

void ArbreDialog::VoirTemp()
{
if (erreurProcessus) return;
QString fileImage=tempDir+"/"+pdfTempFile;
QFileInfo fi(fileImage);
if (fi.exists()) 
	{
	if (pdfviewerWindow)
	  {
	  pdfviewerWindow->openFile(fileImage,lecteurpdfCommande);
	  }
	else
	  {
	  pdfviewerWindow=new PdfViewer(fileImage,lecteurpdfCommande,this);
	  }  
	  pdfviewerWindow->raise();
	  pdfviewerWindow->show();
	}
}

