/***************************************************************************
 *   copyright       : (C) 2012-2021 by Pascal Brachet                     *
 *   https://www.xm1math.net/pdfadd/                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   @license GPL-2.0+ <https://spdx.org/licenses/GPL-2.0+.html>           *
 ***************************************************************************/

#include "binomdialog.h"
#include "aidedialog.h"

#include <QDebug>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QDomNodeList>
#include <QDomProcessingInstruction>
#include <QFile>
#include <QDir>
#include <QMessageBox>
#include <QFileDialog>
#include <QFileInfo>
#include <QUrl>
#include <QFontDatabase>
#include <QProgressBar>
#include <QRegularExpression>
#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
    #include <QtCore/QTextCodec>
#else
    #include <QtCore5Compat/QTextCodec>
#endif


#include <math.h>

BinomDialog::BinomDialog(QWidget *parent)
    : QDialog( parent)
{
ui.setupUi(this);
#if defined(Q_OS_MAC)
setModal(true);
#endif

connect(ui.barreOutils->actionQuitter, SIGNAL(triggered()), this, SLOT(accept()));
connect(ui.barreOutils->actionGenerer, SIGNAL(triggered()), this, SLOT(AfficherCode()));
connect(ui.barreOutils->actionNouveau, SIGNAL(triggered()), this, SLOT(NouvelleBinomiale()));
connect(ui.barreOutils->actionOuvrir, SIGNAL(triggered()), this, SLOT(ChargerBinomiale()));
connect(ui.barreOutils->actionSauver, SIGNAL(triggered()), this, SLOT(SauverBinomiale()));
connect(ui.barreOutils->actionAide, SIGNAL(triggered()), this, SLOT(Aide()));
connect(ui.barreOutils->actionExporterPdf, SIGNAL(triggered()), this, SLOT(ExporterPdf()));
connect(ui.barreOutils->actionApercu, SIGNAL(triggered()), this, SLOT(AsyPdfTemp()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(VoirTemp()));




connect(ui.comboBoxCouleur, SIGNAL(currentTextChanged(const QString &)),this,SLOT(NettoyerCode()));

connect(ui.lineEditN, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditP, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));


connect(ui.lineEditUniteX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditUniteY, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGradX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));



ui.comboBoxCouleur->addItems(QString("couleur1,couleur2,couleur3").split(","));

tempDir=QDir::tempPath();
prefixFile=QDir::homePath();
prefixFile="pdfadd_temp_"+prefixFile.section('/',-1);
prefixFile=QString(QUrl::toPercentEncoding(prefixFile));
prefixFile.remove("%");
asyTempFile=prefixFile+".asy";
pdfTempFile=prefixFile+".pdf";


highlighterSupp= new LatexHighlighter(ui.textEditSupp->document());

QFont efont;
QFontDatabase fdb;
QStringList xf = fdb.families();
QString deft;
if (xf.contains("Liberation Mono",Qt::CaseInsensitive)) deft="Liberation Mono";
else if (xf.contains("DejaVu Sans Mono",Qt::CaseInsensitive)) deft="DejaVu Sans Mono";
#if defined(Q_OS_MAC)
else if (xf.contains("Courier",Qt::CaseInsensitive)) deft="Courier";
#endif
#if defined(Q_OS_WIN32)
else if (xf.contains("Courier New",Qt::CaseInsensitive)) deft="Courier New";
#endif
else deft=qApp->font().family();  
efont.setFamily(deft);
efont.setPointSize(qApp->font().pointSize());
ui.textEditCode->setFont(efont);
ui.textEditSupp->setFont(efont);
ui.textEditCode->setWordWrapMode(QTextOption::NoWrap);
ui.textEditSupp->setWordWrapMode(QTextOption::NoWrap);

Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Loi binomiale : ")+nomFichier);
show();
}

BinomDialog::~BinomDialog(){
// if (asy_pdf_proc && asy_pdf_proc->state()==QProcess::Running)
// 	{
// 	asy_pdf_proc->disconnect();
// 	asy_pdf_proc->kill();
// 	delete asy_pdf_proc ;
// 	}
}

void BinomDialog::closeEvent(QCloseEvent *e)
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Loi binomiale : ",
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverBinomiale();
		break;
		case 1:
		break;
		case 2:
		default:
		e->ignore();
		return;
		break;
		}
	}
e->accept();
}

void BinomDialog::accept()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Loi binomiale : ",
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverBinomiale();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::accept();
}

void BinomDialog::reject()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Loi binomiale : ",
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverBinomiale();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::reject();
}

void BinomDialog::Init()
{
ui.textEditCode->clear();
ui.textEditSupp->clear();
ui.comboBoxCouleur->setCurrentIndex(1);
ui.lineEditUniteX->setText("0.2");
ui.lineEditGradX->setText("2");
ui.lineEditUniteY->setText("25");
ui.lineEditN->setText("20");
ui.lineEditP->setText("0.5");
}
//*********************************************
double BinomDialog::binomiale(int n, int k, double p)
{
double cb=1;
for (int i = 0; i < k; ++i) 
  {
  cb *= n - i;
  cb /= k -i;
  }
return cb*pow(p,k)*pow(1-p,n-k);
}

QString BinomDialog::GenererCode()
{
QString couleur,s_min,s_max,s_hauteur;
bool ok;

QString s_unitX=ui.lineEditUniteX->text();
double unitX;
unitX=s_unitX.toDouble(&ok);


QString s_unitY=ui.lineEditUniteY->text();
double unitY;
unitY=s_unitY.toDouble(&ok);

QString s_Xmin="0";
double Xmin=0;

QString s_Ymin="0";
double Ymin=0;

QString s_Xmax=ui.lineEditN->text();
double Xmax;
Xmax=s_Xmax.toDouble(&ok);

int binomN=ui.lineEditN->text().toInt(&ok);
double binomP=ui.lineEditP->text().toFloat(&ok);

double Ymax=0;

ui.textEditSupp->clear();

QString supcode="\\begin{small}\n\\begin{tabular}{|c|c|}\n";
supcode+="\\hline $k$ & $p(X = k)$ \\\\ \n";
QString supcodebis="%cumul\n\\begin{small}\n\\begin{tabular}{|c|c|}\n";
supcodebis+="\\hline $k$ & $p(X \\leqslant k)$ \\\\ \n";

QString tabul, cumul;
double pk;
double cum=0;
for (int k = 0; k <= binomN; ++k)
  {
  pk=binomiale(binomN, k,binomP);
  tabul=QString().asprintf ("%.8f",pk);
  supcode+="\\hline "+QString::number(k)+" & "+tabul+" \\\\ \n";
  cum+=pk;
  cumul=QString().asprintf ("%.8f",cum);
  supcodebis+="\\hline "+QString::number(k)+" & "+cumul+" \\\\ \n";
  if (pk>Ymax) Ymax=pk;
  }
supcode+="\\hline\n";
supcode+="\\end{tabular}\n\\end{small}\n";
supcodebis+="\\hline\n";
supcodebis+="\\end{tabular}\n\\end{small}\n";
ui.textEditSupp->setPlainText("%\\usepackage{amsmath,amsfonts,amssymb}\nLoi binomiale pour $n="+ui.lineEditN->text()+"$ et $p="+ui.lineEditP->text()+"$.\n\n"+supcode+"\n\\medskip\n\n"+supcodebis);

QString s_Ymax=QString::number(Ymax);
QString s_gradX=ui.lineEditGradX->text();
double gradX;
gradX=s_gradX.toDouble(&ok);

double gradY=0.02;
if (Ymax>0.5) gradY=0.1;
QString s_gradY=QString::number(gradY);


QString s_Ox="0";
QString s_Oy="0";

QString texcode=QString::fromUtf8("// préambule asymptote\n")+preambuleAsy+"\n// code figure\n";
texcode+="import graph;\n";
texcode+="unitsize("+s_unitX+"cm,"+s_unitY+"cm);\n";
texcode+="xlimits("+s_Xmin+","+s_Xmax+");\n";
texcode+="ylimits("+s_Ymin+","+s_Ymax+");\n";
QString ticksX=",Ticks(scale(.7)*Label(),beginlabel=true,endlabel=true,begin=true,end=true,Step="+s_gradX+",Size=1mm)";
QString ticksY=",Ticks(scale(.7)*Label(),beginlabel=true,endlabel=true,begin=true,end=true,Step="+s_gradY+",Size=1mm)";
texcode+="xaxis(axis=YEquals("+s_Oy+"),xmin="+s_Xmin+",xmax="+s_Xmax+ticksX+",p=linewidth(1pt)+black,Arrow(2mm),true);\n";
texcode+="yaxis(axis=XEquals("+s_Ox+"),ymin="+s_Ymin+",ymax="+s_Ymax+ticksY+",p=linewidth(1pt)+black,Arrow(2mm),true);\n";
texcode+="labelx(scale(.7)*Label(\"$k$\",NoFill),"+s_Xmax+", E);\n";
texcode+="labely(scale(.7)*Label(\"$p(X=k)$\",NoFill),"+s_Ymax+", N);\n";
int pR1= (couleurSupp1.red()*10000/255);
double cR1=(double) pR1/10000;
int pG1= (couleurSupp1.green()*10000/255);
double cG1=(double) pG1/10000;
int pB1= (couleurSupp1.blue()*10000/255);
double cB1=(double) pB1/10000;
QString color1="rgb("+QString::number(cR1 ,'g',5)+","+QString::number(cG1 ,'g',5)+","+QString::number(cB1 ,'g',5)+")";

int pR2= (couleurSupp2.red()*10000/255);
double cR2=(double) pR2/10000;
int pG2= (couleurSupp2.green()*10000/255);
double cG2=(double) pG2/10000;
int pB2= (couleurSupp2.blue()*10000/255);
double cB2=(double) pB2/10000;
QString color2="rgb("+QString::number(cR2 ,'g',5)+","+QString::number(cG2 ,'g',5)+","+QString::number(cB2 ,'g',5)+")";

int pR3= (couleurSupp3.red()*10000/255);
double cR3=(double) pR3/10000;
int pG3= (couleurSupp3.green()*10000/255);
double cG3=(double) pG3/10000;
int pB3= (couleurSupp3.blue()*10000/255);
double cB3=(double) pB3/10000;
QString color3="rgb("+QString::number(cR3 ,'g',5)+","+QString::number(cG3 ,'g',5)+","+QString::number(cB3 ,'g',5)+")"; 

if (ui.comboBoxCouleur->currentText()=="noir") couleur="black";
else if (ui.comboBoxCouleur->currentText()=="gris") couleur="gray";
else if (ui.comboBoxCouleur->currentText()==QString::fromUtf8("gris foncé")) couleur="heavygray";
else if (ui.comboBoxCouleur->currentText()=="gris clair") couleur="mediumgray";
else if (ui.comboBoxCouleur->currentText()=="bleu") couleur="blue";
else if (ui.comboBoxCouleur->currentText()=="rouge") couleur="red";
else if (ui.comboBoxCouleur->currentText()=="vert") couleur="green";
else if (ui.comboBoxCouleur->currentText()=="jaune") couleur="yellow";
else if (ui.comboBoxCouleur->currentText()=="cyan") couleur="cyan";
else if (ui.comboBoxCouleur->currentText()=="magenta") couleur="magenta";
else if (ui.comboBoxCouleur->currentText()=="couleur1") couleur=color1;
else if (ui.comboBoxCouleur->currentText()=="couleur2") couleur=color2;
else if (ui.comboBoxCouleur->currentText()=="couleur3") couleur=color3;
else couleur="black";
texcode+="int n="+ui.lineEditN->text()+";\n";
texcode+="real p="+ui.lineEditP->text()+";\n";
texcode+="for (int k=0; k<=n; ++k)\n";
texcode+="{\n";
texcode+="draw((k,0)--(k,choose(n,k)*p^k*(1-p)^(n-k)),2bp+"+couleur+"+squarecap);\n";
texcode+="}\n";
texcode+="shipout(bbox(0.1cm,0.1cm,white));\n";
return texcode;
}

void BinomDialog::AfficherCode()
{
ui.textEditCode->clear();
QString texcode=GenererCode()+"\n";
ui.textEditCode->setPlainText(texcode);
}

//*********************************************************************

//******************************************************
void BinomDialog::NouvelleBinomiale()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Loi binomiale : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant d'en créer un nouveau?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverBinomiale();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Loi binomiale : ")+nomFichier);
}
void BinomDialog::SauverBinomiale()
{
QDomDocument doc;
QDomElement description;
QDomText contenu;
QDomProcessingInstruction instr =  doc.createProcessingInstruction("xml","version=\"1.0\" encoding=\"UTF-8\"");
doc.appendChild(instr);
QDomElement root=doc.createElement("Binomiale");
doc.appendChild(root);

description=doc.createElement("Population");
contenu=doc.createTextNode(ui.lineEditN->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Proba");
contenu=doc.createTextNode(ui.lineEditP->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("UnitX");
contenu=doc.createTextNode(ui.lineEditUniteX->text());
root.appendChild(description);
description.appendChild(contenu);


description=doc.createElement("UnitY");
contenu=doc.createTextNode(ui.lineEditUniteY->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GradX");
contenu=doc.createTextNode(ui.lineEditGradX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Couleur");
contenu=doc.createTextNode(ui.comboBoxCouleur->currentText());
root.appendChild(description);
description.appendChild(contenu);

QString nouveauFichier;
if (nomFichier=="sanstitre" ) 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Binomiale (*.pag)"));
	}
else 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),nomFichier,QString::fromUtf8("Binomiale (*.pag)"));
	}
//nouveauFichier=nomFichier;
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::WriteOnly))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible d'enregistrer le fichier"));
	return;
	}
nomFichier=nouveauFichier;
QTextStream out (&fichier);
doc.save(out,4);
fichier.close();
QFileInfo fi(nomFichier);
dernierRepertoire=fi.absolutePath();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Loi binomiale : ")+nomFichier);
}

void BinomDialog::Ouvrir(QString nouveauFichier)
{
QDomDocument doc;
QDomNodeList nodelist;
QDomNode node;
QFileInfo fi(nouveauFichier);
if (!fi.exists()) return;
dernierRepertoire=fi.absolutePath();
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::ReadOnly)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	return;
	}
if (!doc.setContent(&fichier)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	fichier.close();
	return;
	}

QDomElement root = doc.documentElement();
if (root.tagName() != "Binomiale")
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Ce fichier ne correspond pas à une loi binomiale"));
	fichier.close();
	return;
	}
nomFichier=nouveauFichier;
Init();

nodelist = doc.elementsByTagName("Population");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditN->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Proba");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditP->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("UnitX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditUniteX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("UnitY");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditUniteY->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GradX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGradX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Couleur");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.comboBoxCouleur->setCurrentIndex(ui.comboBoxCouleur->findText(node.toElement().text(),Qt::MatchExactly));
	}

AfficherCode();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Loi binomiale : ")+nomFichier);
}

void BinomDialog::ChargerBinomiale()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Loi binomiale : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de charger un autre graphe?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverBinomiale();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QString nouveauFichier = QFileDialog::getOpenFileName(this,QString::fromUtf8("Ouvrir Fichier"),dernierRepertoire,"Binomiale (*.pag)");
if (nouveauFichier.isEmpty()) return;
Ouvrir(nouveauFichier);
}
//*********************************************
void BinomDialog::NettoyerCode()
{
ui.textEditCode->clear();
ui.textEditSupp->clear();
estModifie=true;
setWindowTitle(QString::fromUtf8("Assistant Loi binomiale ")+QString::fromUtf8("[modifié]")+" : "+nomFichier);
}

//*******************************
void BinomDialog::Aide()
{
AideDialog *aideDialog = new AideDialog(this,"binomiale");
aideDialog->exec();
}
//*****************************
void BinomDialog::ExporterPdf()
{
this->disconnect(SIGNAL(asy_pdf_fini()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(ExporterPdfSuite()));
AsyPdfTemp();
}

void BinomDialog::ExporterPdfSuite()
{
if (!erreurProcessus)
	{
	QString pdfFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Pdf (*.pdf)"));
	if (!pdfFichier.isEmpty())
		{
		QFileInfo fi(pdfFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+pdfTempFile,pdfFichier);
		}
	}
this->disconnect(SIGNAL(asy_pdf_fini()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(VoirTemp()));
}

void BinomDialog::CopierFichier(QString origine,QString destination)
{
if (destination.isEmpty() || origine.isEmpty()) return;
QFileInfo fi_or(origine);
if (!fi_or.exists()) return;
QFile fichier_or(origine);
QFileInfo fi_dest(destination);
if (fi_dest.exists())
	{
	QFile fichier_dest(destination);
	fichier_dest.remove();
	fichier_or.copy(destination);
	}
else
	{
	fichier_or.copy(destination);
	}
}

void BinomDialog::AsyPdfTemp()
{
AfficherCode();
if (ui.textEditCode->toPlainText().contains(QRegularExpression("Erreur|erreur")))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Erreurs dans le code généré.\nLa compilation n'est pas lancée."));
	return;
	}
QFile fichier;
fichier.setFileName(tempDir+"/"+asyTempFile);
fichier.open(QIODevice::WriteOnly);
QTextCodec *codec = QTextCodec::codecForName("UTF-8");
fichier.write(codec->fromUnicode(GenererCode()));
fichier.close();


erreurProcessus=false;
asy_pdf_proc = new CmdLauncher( this, tempDir, extraPath );
connect(asy_pdf_proc, SIGNAL(jobdone()),this, SIGNAL(asy_pdf_fini()));
connect(asy_pdf_proc, SIGNAL(joberror()),this, SLOT(ErreurCompilation()));
asy_pdf_proc->launchCommand(asyCommande+" -f pdf -noView "+asyTempFile);
}




void BinomDialog::ErreurCompilation()
{
erreurProcessus=true;
}

void BinomDialog::VoirTemp()
{
if (erreurProcessus) return;
QString fileImage=tempDir+"/"+pdfTempFile;
QFileInfo fi(fileImage);
if (fi.exists()) 
	{
	if (pdfviewerWindow)
	  {
	  pdfviewerWindow->openFile(fileImage,lecteurpdfCommande);
	  }
	else
	  {
	  pdfviewerWindow=new PdfViewer(fileImage,lecteurpdfCommande,this);
	  }  
	  pdfviewerWindow->raise();
	  pdfviewerWindow->show();
	}
}
