/***************************************************************************
 *   copyright       : (C) 2012-2021 by Pascal Brachet                     *
 *   https://www.xm1math.net/pdfadd/                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   @license GPL-2.0+ <https://spdx.org/licenses/GPL-2.0+.html>           *
 ***************************************************************************/
#include "cmdlauncher.h"
#include "cmdparser.h"


#include <QMessageBox>
#include <QProgressBar>

CmdLauncher::CmdLauncher(QObject *parent, const QString &workdir, const QString &extrapath) : QObject(parent)
{
process = new QProcess(this);
message="";
err_detected=false;
process->setWorkingDirectory(workdir);
QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
#if defined(Q_OS_MAC)
env.insert("PATH", env.value("PATH") + extrapath);
process->setProcessEnvironment(env);
#endif
#if defined(Q_OS_WIN32)
env.insert("PATH", env.value("PATH") + extrapath);
#endif
qputenv("PATH", env.value("PATH").toLatin1());
connect(process, SIGNAL(finished(int)),this, SLOT(endProcess(int)));
connect(process, SIGNAL(readyReadStandardError()),this, SLOT(readInfoOutput()));
}

CmdLauncher::~CmdLauncher(){
if (progressDialog) delete progressDialog;
if (process && process->state()==QProcess::Running)
	{
	process->disconnect();
	process->kill();
	delete process ;
	}
}

void CmdLauncher::launchCommand(const QString &command)
{
QStringList args = splitCommand(command);
const QString program = args.takeFirst();
process->start(program, args);

if (!process->waitForStarted(1000))
	{
    QMessageBox::warning( static_cast<QWidget*>(parent()),QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lancer la commande ")+program+"\n"+message);
	err_detected=true;
    emit joberror();
    }
else
    {
    if (!progressDialog) progressDialog = new QProgressDialog(static_cast<QWidget*>(parent()));
    QProgressBar* progress = new QProgressBar();
    progress->setFormat(" ");
    progress->setMaximum(0);
    progress->setMinimum(0);
    progressDialog->setBar(progress);
    progressDialog->setLabelText(QString::fromUtf8("Compilation %1 en cours... ").arg(program));
    progressDialog->setCancelButton(0);
    progressDialog->setRange(0,0);
    progressDialog->setMinimumDuration(0);
    progressDialog->show();
    progressDialog->raise();
    }
}

void CmdLauncher::readInfoOutput()
{
QByteArray result=process->readAllStandardError();
message=QString(result);
message=message.simplified();
}

void CmdLauncher::endProcess(int err)
{
if (progressDialog) progressDialog->cancel();
if (err)
	{
	QMessageBox::warning( static_cast<QWidget*>(parent()),QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la compilation ")+"\n"+message);
	err_detected=true;
    emit joberror();
	}
else
	{
	emit jobdone();
	}
}
