/***************************************************************************
 *   copyright       : (C) 2012-2021 by Pascal Brachet                     *
 *   https://www.xm1math.net/pdfadd/                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   @license GPL-2.0+ <https://spdx.org/licenses/GPL-2.0+.html>           *
 ***************************************************************************/

#include "espacedialog.h"
#include "aidedialog.h"
#include "logdialog.h"

#include <QDebug>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QDomNodeList>
#include <QDomProcessingInstruction>
#include <QFile>
#include <QDir>
#include <QMessageBox>
#include <QFileDialog>
#include <QFileInfo>
#include <QUrl>
#include <QFontDatabase>
#include <QRegularExpression>
#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
    #include <QtCore/QTextCodec>
#else
    #include <QtCore5Compat/QTextCodec>
#endif


#include <math.h>

EspaceDialog::EspaceDialog(QWidget *parent)
    :QDialog( parent)
{
ui.setupUi(this);
#if defined(Q_OS_MAC)
setModal(true);
#endif
connect(ui.barreOutils->actionQuitter, SIGNAL(triggered()), this, SLOT(accept()));
connect(ui.barreOutils->actionGenerer, SIGNAL(triggered()), this, SLOT(AfficherCode()));
connect(ui.barreOutils->actionNouveau, SIGNAL(triggered()), this, SLOT(NouvelEspace()));
connect(ui.barreOutils->actionOuvrir, SIGNAL(triggered()), this, SLOT(ChargerEspace()));
connect(ui.barreOutils->actionSauver, SIGNAL(triggered()), this, SLOT(SauverEspace()));
connect(ui.barreOutils->actionAide, SIGNAL(triggered()), this, SLOT(Aide()));
connect(ui.barreOutils->actionExporterPdf, SIGNAL(triggered()), this, SLOT(ExporterPdf()));
connect(ui.barreOutils->actionApercu, SIGNAL(triggered()), this, SLOT(AsyPdfTemp()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(VoirTemp()));

connect(ui.listWidgetObjets, SIGNAL(itemSelectionChanged()),this, SLOT(actualiserItemInfo()));
connect(ui.lineEditObjet, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserExpression()));
connect(ui.comboBoxCouleur, SIGNAL(currentTextChanged(const QString &)),this,SLOT(actualiserCouleur()));
connect(ui.comboBoxStyle, SIGNAL(currentTextChanged(const QString &)),this,SLOT(actualiserStyle()));
connect(ui.comboBoxType, SIGNAL(currentTextChanged(const QString &)),this,SLOT(actualiserType()));
connect(ui.comboBoxRemplissage, SIGNAL(currentTextChanged(const QString &)),this,SLOT(actualiserRemplissage()));
connect(ui.ajouterButton, SIGNAL(clicked()), this, SLOT(ajouterObjet()));
connect(ui.supprimerButton, SIGNAL(clicked()), this, SLOT(supprimerObjet()));
connect(ui.remonterButton, SIGNAL(clicked()), this, SLOT(remonterObjet()));
connect(ui.descendreButton, SIGNAL(clicked()), this, SLOT(descendreObjet()));


connect(ui.lineEditUnite, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditCoeff, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.spinBoxXmin, SIGNAL(valueChanged(int)),this, SLOT(NettoyerCode()));
connect(ui.spinBoxXmax, SIGNAL(valueChanged(int)),this, SLOT(NettoyerCode()));
connect(ui.spinBoxYmin, SIGNAL(valueChanged(int)),this, SLOT(NettoyerCode()));
connect(ui.spinBoxYmax, SIGNAL(valueChanged(int)),this, SLOT(NettoyerCode()));
connect(ui.spinBoxZmin, SIGNAL(valueChanged(int)),this, SLOT(NettoyerCode()));
connect(ui.spinBoxZmax, SIGNAL(valueChanged(int)),this, SLOT(NettoyerCode()));

connect(ui.textEditSupp, SIGNAL(textChanged()),this,SLOT(NettoyerCode()));

connect(ui.checkBoxLabels, SIGNAL(toggled(bool)),this, SLOT(NettoyerCode()));

connect(ui.pushButtonTranslater, SIGNAL(clicked()), this, SLOT(Translater()));



ui.listWidgetObjets->setAlternatingRowColors(true);
ui.listWidgetObjets->setSelectionMode (QAbstractItemView::SingleSelection);

ui.comboBoxCouleur->addItems(QString("couleur1,couleur2,couleur3").split(","));

tempDir=QDir::tempPath();
prefixFile=QDir::homePath();
prefixFile="pdfadd_temp_"+prefixFile.section('/',-1);
prefixFile=QString(QUrl::toPercentEncoding(prefixFile));
prefixFile.remove("%");
asyTempFile=prefixFile+".asy";
pdfTempFile=prefixFile+".pdf";

highlighterSupp = new Highlighter(ui.textEditSupp->document());

QFont efont;
QFontDatabase fdb;
QStringList xf = fdb.families();
QString deft;
if (xf.contains("Liberation Mono",Qt::CaseInsensitive)) deft="Liberation Mono";
else if (xf.contains("DejaVu Sans Mono",Qt::CaseInsensitive)) deft="DejaVu Sans Mono";
#if defined(Q_OS_MAC)
else if (xf.contains("Courier",Qt::CaseInsensitive)) deft="Courier";
#endif
#if defined(Q_OS_WIN32)
else if (xf.contains("Courier New",Qt::CaseInsensitive)) deft="Courier New";
#endif
else deft=qApp->font().family();  
efont.setFamily(deft);
efont.setPointSize(qApp->font().pointSize());
ui.textEditCode->setFont(efont);
ui.textEditSupp->setFont(efont);
ui.textEditCode->setWordWrapMode(QTextOption::NoWrap);
ui.textEditSupp->setWordWrapMode(QTextOption::NoWrap);
ui.lineEditObjet->setFont(efont);

Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Espace : ")+nomFichier);
show();
}

EspaceDialog::~EspaceDialog(){
// if (asy_pdf_proc && asy_pdf_proc->state()==QProcess::Running)
// 	{
// 	asy_pdf_proc->disconnect();
// 	asy_pdf_proc->kill();
// 	delete asy_pdf_proc ;
// 	}
}

void EspaceDialog::closeEvent(QCloseEvent *e)
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Espace : ",
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverEspace();
		break;
		case 1:
		break;
		case 2:
		default:
		e->ignore();
		return;
		break;
		}
	}
e->accept();
}

void EspaceDialog::accept()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Espace : ",
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverEspace();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::accept();
}

void EspaceDialog::reject()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Espace : ",
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverEspace();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::reject();
}

void EspaceDialog::Init()
{
ui.textEditCode->clear();
expressionList.clear();
couleurList.clear();
styleList.clear();
typeList.clear();
remplissageList.clear();
ui.listWidgetObjets->clear();
ui.checkBoxLabels->setChecked(true);
ui.lineEditUnite->setText("1");
ui.lineEditCoeff->setText("0.5");
ui.spinBoxXmin->setValue(0);
ui.spinBoxXmax->setValue(3);
ui.spinBoxYmin->setValue(0);
ui.spinBoxYmax->setValue(3);
ui.spinBoxZmin->setValue(0);
ui.spinBoxZmax->setValue(3);

ui.textEditSupp->clear();
ActiverObjets(false);
}
//*********************************************
void EspaceDialog::ActiverObjets(bool etat)
{
ui.lineEditObjet->setEnabled(etat);
ui.comboBoxCouleur->setEnabled(etat);
ui.comboBoxStyle->setEnabled(etat);
ui.comboBoxType->setEnabled(etat);
ui.comboBoxRemplissage->setEnabled(etat);
//NettoyerCode();
}
//******************************************************************
QString EspaceDialog::GenererCode()
{
bool ok;

QString s_unitGraph=ui.lineEditUnite->text();
double unitGraph=s_unitGraph.toDouble(&ok);
if (unitGraph<=0) unitGraph=1;

QString s_coeff=ui.lineEditCoeff->text();
double coeff=s_coeff.toDouble(&ok);

int Xmin=ui.spinBoxXmin->value();
QString s_Xmin=QString::number(Xmin);

int Xmax=ui.spinBoxXmax->value();
QString s_Xmax=QString::number(Xmax);

int Ymin=ui.spinBoxYmin->value();
QString s_Ymin=QString::number(Ymin);

int Ymax=ui.spinBoxYmax->value();
QString s_Ymax=QString::number(Ymax);

int Zmin=ui.spinBoxZmin->value();
QString s_Zmin=QString::number(Zmin);

int Zmax=ui.spinBoxZmax->value();
QString s_Zmax=QString::number(Zmax);

double decal=0.5;

double x1=(double) Ymin-decal-coeff*(Xmin-decal);
double y1=(double) Zmin-decal-coeff*(Xmin-decal);

double x2=(double) Ymin-decal-coeff*(Xmax+decal);
double y2=(double) Zmin-decal-coeff*(Xmax+decal);

double cadreXmin=qMin(x1,x2);
double cadreYmin=qMin(y1,y2);
double cadreXmax=qMax(x1,x2);
double cadreYmax=qMax(y1,y2);

double x3=(double) Ymax+decal-coeff*(Xmax+decal);
double y3=(double) Zmin-decal-coeff*(Xmax+decal);

cadreXmin=qMin(cadreXmin,x3);
cadreYmin=qMin(cadreYmin,y3);
cadreXmax=qMax(cadreXmax,x3);
cadreYmax=qMax(cadreYmax,y3);

double x4=(double) Ymax+decal-coeff*(Xmin-decal);
double y4=(double) Zmin-decal-coeff*(Xmin-decal);

cadreXmin=qMin(cadreXmin,x4);
cadreYmin=qMin(cadreYmin,y4);
cadreXmax=qMax(cadreXmax,x4);
cadreYmax=qMax(cadreYmax,y4);

double x5=(double) Ymin-decal-coeff*(Xmin-decal);
double y5=(double) Zmax+decal-coeff*(Xmin-decal);

cadreXmin=qMin(cadreXmin,x5);
cadreYmin=qMin(cadreYmin,y5);
cadreXmax=qMax(cadreXmax,x5);
cadreYmax=qMax(cadreYmax,y5);

double x6=(double) Ymin-decal-coeff*(Xmax+decal);
double y6=(double) Zmax+decal-coeff*(Xmax+decal);

cadreXmin=qMin(cadreXmin,x6);
cadreYmin=qMin(cadreYmin,y6);
cadreXmax=qMax(cadreXmax,x6);
cadreYmax=qMax(cadreYmax,y6);

double x7=(double) Ymax+decal-coeff*(Xmax+decal);
double y7=(double) Zmax+decal-coeff*(Xmax+decal);

cadreXmin=qMin(cadreXmin,x7);
cadreYmin=qMin(cadreYmin,y7);
cadreXmax=qMax(cadreXmax,x7);
cadreYmax=qMax(cadreYmax,y7);

double x8=(double) Ymax+decal-coeff*(Xmin-decal);
double y8=(double) Zmax+decal-coeff*(Xmin-decal);

cadreXmin=qMin(cadreXmin,x8);
cadreYmin=qMin(cadreYmin,y8);
cadreXmax=qMax(cadreXmax,x8);
cadreYmax=qMax(cadreYmax,y8);

QString texcode=QString::fromUtf8("// préambule asymptote\n")+preambuleAsy+"\n// code figure\n";
texcode+="import three;\n";
texcode+="import patterns;\n";
texcode+="projection caval(real k=.5, real angle=45)\n";
texcode+="{\n";
texcode+="transform3 t=identity(4);\n";
texcode+="real c2=Cos(angle)^2;\n";
texcode+="real s2=1-c2;\n";
texcode+="t[0][0]=-k*Cos(angle);\n";
texcode+="t[1][0]=-k*Sin(angle);\n";
texcode+="t[1][1]=0;\n";
texcode+="t[0][1]=1;\n";
texcode+="t[1][2]=1;\n";
texcode+="t[2][2]=0;\n";
texcode+="t[2][0]=1;\n";
texcode+="t[2][3]=-1;\n";
texcode+="return projection((1,c2,s2),normal=(1,0,0),\n";
texcode+="new transformation(triple,triple,triple) {\n";
texcode+="return transformation(t);});\n";
texcode+="}\n\n";
texcode+="settings.render=0;\n";
texcode+="settings.prc=false;\n";
texcode+="currentlight=nolight;\n";
texcode+="unitsize("+s_unitGraph+"cm);\n";
texcode+="currentprojection=caval("+s_coeff+",45);\n";
if (ui.checkBoxLabels->isChecked())
  {
  texcode+="draw(Label(\"$x$\",EndPoint,NW),("+s_Xmin+",0,0)--("+s_Xmax+",0,0),black,Arrow3);\n";
  texcode+="for (int ix ="+s_Xmin+"; ix <"+s_Xmax+"; ++ix)\n";
  texcode+="{\n";
  texcode+="if (ix!=0) draw(Label(scale(0.7)*((string) ix),BeginPoint,NW),(ix,-0.1,0)--(ix,0.1,0),darkgray);\n";
  texcode+="}\n";
  texcode+="draw(Label(\"$y$\",EndPoint,N),(0,"+s_Ymin+",0)--(0,"+s_Ymax+",0),black,Arrow3);\n";
  texcode+="for (int iy ="+s_Ymin+"; iy <"+s_Ymax+"; ++iy)\n";
  texcode+="{\n";
  texcode+="if (iy!=0) draw(Label(scale(0.7)*((string) iy),BeginPoint,N),(-0.1,iy,0)--(0.1,iy,0),darkgray);\n";
  texcode+="}\n";
  texcode+="draw(Label(\"$z$\",EndPoint,W),(0,0,"+s_Zmin+")--(0,0,"+s_Zmax+"),black,Arrow3);\n";
  texcode+="for (int iz ="+s_Zmin+"; iz <"+s_Zmax+"; ++iz)\n";
  texcode+="{\n";
  texcode+="if (iz!=0) draw(Label(scale(0.7)*((string) iz),BeginPoint,W),(0,-0.1,iz)--(0,0.1,iz),darkgray);\n";
  texcode+="}\n";
  }

int pR1= (couleurSupp1.red()*10000/255);
double cR1=(double) pR1/10000;
int pG1= (couleurSupp1.green()*10000/255);
double cG1=(double) pG1/10000;
int pB1= (couleurSupp1.blue()*10000/255);
double cB1=(double) pB1/10000;
QString color1="rgb("+QString::number(cR1 ,'g',5)+","+QString::number(cG1 ,'g',5)+","+QString::number(cB1 ,'g',5)+")";

int pR2= (couleurSupp2.red()*10000/255);
double cR2=(double) pR2/10000;
int pG2= (couleurSupp2.green()*10000/255);
double cG2=(double) pG2/10000;
int pB2= (couleurSupp2.blue()*10000/255);
double cB2=(double) pB2/10000;
QString color2="rgb("+QString::number(cR2 ,'g',5)+","+QString::number(cG2 ,'g',5)+","+QString::number(cB2 ,'g',5)+")";

int pR3= (couleurSupp3.red()*10000/255);
double cR3=(double) pR3/10000;
int pG3= (couleurSupp3.green()*10000/255);
double cG3=(double) pG3/10000;
int pB3= (couleurSupp3.blue()*10000/255);
double cB3=(double) pB3/10000;
QString color3="rgb("+QString::number(cR3 ,'g',5)+","+QString::number(cG3 ,'g',5)+","+QString::number(cB3 ,'g',5)+")";

QString couleur,style,remplissage;
QStringList val,points,sommets;
QString xpmin,ypmin,zpmin,xpmax,ypmax,zpmax;
for (int i = 0; i < expressionList.count(); i++)
  {
  if (couleurList.at(i)=="noir") couleur="black";
  else if (couleurList.at(i)=="gris") couleur="gray";
  else if (couleurList.at(i)==QString::fromUtf8("gris foncé")) couleur="heavygray";
  else if (couleurList.at(i)=="gris clair") couleur="mediumgray";
  else if (couleurList.at(i)=="bleu") couleur="blue";
  else if (couleurList.at(i)=="rouge") couleur="red";
  else if (couleurList.at(i)=="vert") couleur="green";
  else if (couleurList.at(i)=="jaune") couleur="yellow";
  else if (couleurList.at(i)=="cyan") couleur="cyan";
  else if (couleurList.at(i)=="magenta") couleur="magenta";
  else if (couleurList.at(i)=="couleur1") couleur=color1;
  else if (couleurList.at(i)=="couleur2") couleur=color2;
  else if (couleurList.at(i)=="couleur3") couleur=color3;
  else if (couleurList.at(i)=="blanc") couleur="white";
  else couleur="black";
  if (styleList.at(i)=="ligne") style="solid";
  else if (styleList.at(i)=="trait") style="dashed";
  else if (styleList.at(i)=="point") style="dotted";
  else style="solid";
  if (typeList.at(i)=="Point")
    {
    val=expressionList.at(i).split(";");
    if (val.count()==3)
      {
      texcode+="dot(("+val.at(0)+","+val.at(1)+","+val.at(2)+"),"+couleur+");\n";
      }
    }
  else if (typeList.at(i)=="Point avec traces")
    {
    val=expressionList.at(i).split(";");
    if (val.count()==3)
      {
      texcode+="dot(("+val.at(0)+","+val.at(1)+","+val.at(2)+"),"+couleur+");\n";
      texcode+="draw(("+val.at(0)+",0,0)--("+val.at(0)+","+val.at(1)+",0)--(0,"+val.at(1)+",0),"+couleur+"+dashed);\n";
      texcode+="draw((0,0,0)--("+val.at(0)+","+val.at(1)+",0)--("+val.at(0)+","+val.at(1)+","+val.at(2)+")--(0,0,"+val.at(2)+"),"+couleur+"+dashed);\n";
      }
    }
  else if (typeList.at(i)=="Segment")
    {
    points=expressionList.at(i).split(":");
    if (points.count()==2)
      {
      texcode+="draw(";
      val=points.at(0).split(";");
      texcode+="("+val.at(0)+","+val.at(1)+","+val.at(2)+")--";
      val=points.at(1).split(";");
      texcode+="("+val.at(0)+","+val.at(1)+","+val.at(2)+"),"+couleur+"+"+style+");\n";
      }
    }
  else if (typeList.at(i)=="Vecteur")
    {
    points=expressionList.at(i).split(":");
    if (points.count()==2)
      {
      texcode+="draw(";
      val=points.at(0).split(";");
      texcode+="("+val.at(0)+","+val.at(1)+","+val.at(2)+")--";
      val=points.at(1).split(";");
      texcode+="("+val.at(0)+","+val.at(1)+","+val.at(2)+"),"+couleur+"+"+style+",Arrow3);\n";
      }
    }
  else if (typeList.at(i)=="Polygone")
    {
    points=expressionList.at(i).split(":");
    if (points.count()>0)
      {
     texcode+="path3 polygone"+QString::number(i)+"=";
      for (int j = 0; j < points.count()-1; j++)
	{
	val=points.at(j).split(";");
	texcode+="("+val.at(0)+","+val.at(1)+","+val.at(2)+")--";
	}
      val=points.at(points.count()-1).split(";");
      texcode+="("+val.at(0)+","+val.at(1)+","+val.at(2)+")--cycle;\n";
      if (remplissageList.at(i)=="plein") 
      {
      texcode+="filldraw(project(polygone"+QString::number(i)+"),"+couleur+","+couleur+");\n";
      }
      else if (remplissageList.at(i)=="lignes") 
      {
      texcode+="add(\"hachure"+QString::number(i)+"\",hatch(dir=SE,"+couleur+"));\n";
      texcode+="filldraw(project(polygone"+QString::number(i)+"),pattern(\"hachure"+QString::number(i)+"\"),"+couleur+");\n";	
      }
      else if (remplissageList.at(i)=="hachures") 
      {
      texcode+="add(\"hachure"+QString::number(i)+"\",crosshatch("+couleur+"));\n";
      texcode+="filldraw(project(polygone"+QString::number(i)+"),pattern(\"hachure"+QString::number(i)+"\"),"+couleur+");\n";	
      }
      else 
      {
      texcode+="draw(polygone"+QString::number(i)+","+couleur+"+"+style+");\n";
      }
      }
      
    }
  else if (typeList.at(i)=="Label")
    {
    val=expressionList.at(i).split(";");
    if (val.count()==5)
      {
      texcode+="label(\""+val.at(4)+"\",("+val.at(0)+","+val.at(1)+","+val.at(2)+"),Relative(dir("+val.at(3)+")));\n";
      }
    }
  else if (typeList.at(i)==QString::fromUtf8("Pavé droit"))
    {
    points=expressionList.at(i).split(":");
if (points.count()==2)
      {
      val=points.at(0).split(";");
      if (val.count()==3)
	  {
	  xpmin=val.at(0);
	  ypmin=val.at(1);
	  zpmin=val.at(2);
	  val=points.at(1).split(";");
	  if (val.count()==3)
	      {
	      xpmax=val.at(0);
	      ypmax=val.at(1);
	      zpmax=val.at(2);
	      sommets.clear();
	      sommets.append(xpmin+";"+ypmin+";"+zpmax);
	      sommets.append(xpmax+";"+ypmin+";"+zpmax);
	      sommets.append(xpmax+";"+ypmax+";"+zpmax);
	      sommets.append(xpmin+";"+ypmax+";"+zpmax);
	      sommets.append(xpmin+";"+ypmin+";"+zpmax);
	      sommets.append(xpmax+";"+ypmin+";"+zpmax);
	      sommets.append(xpmax+";"+ypmin+";"+zpmin);
	      sommets.append(xpmax+";"+ypmax+";"+zpmin);
	      sommets.append(xpmax+";"+ypmax+";"+zpmax);
	      sommets.append(xpmin+";"+ypmax+";"+zpmax);
	      sommets.append(xpmin+";"+ypmax+";"+zpmin);
	      sommets.append(xpmax+";"+ypmax+";"+zpmin);
	      if (sommets.count()>0)
		{
		texcode+="draw(";
		  for (int j = 0; j < sommets.count()-1; j++)
		    {
		    val=sommets.at(j).split(";");
		    texcode+="("+val.at(0)+","+val.at(1)+","+val.at(2)+")--";
		    }
		  val=sommets.at(sommets.count()-1).split(";");
		  texcode+="("+val.at(0)+","+val.at(1)+","+val.at(2)+"),"+couleur+"+"+style+");\n";		  
		}
	      sommets.clear();
	      sommets.append(xpmax+";"+ypmin+";"+zpmin);
	      sommets.append(xpmin+";"+ypmin+";"+zpmin);
	      sommets.append(xpmin+";"+ypmax+";"+zpmin);
	      if (sommets.count()>0)
		{
		texcode+="draw(";
		  for (int j = 0; j < sommets.count()-1; j++)
		    {
		    val=sommets.at(j).split(";");
		    texcode+="("+val.at(0)+","+val.at(1)+","+val.at(2)+")--";
		    }
		  val=sommets.at(sommets.count()-1).split(";");
		  texcode+="("+val.at(0)+","+val.at(1)+","+val.at(2)+"),"+couleur+"+dashed);\n";
		}
	      sommets.clear();
	      sommets.append(xpmin+";"+ypmin+";"+zpmin);
	      sommets.append(xpmin+";"+ypmin+";"+zpmax);
	      if (sommets.count()>0)
		  {
		texcode+="draw(";
		  for (int j = 0; j < sommets.count()-1; j++)
		    {
		    val=sommets.at(j).split(";");
		    texcode+="("+val.at(0)+","+val.at(1)+","+val.at(2)+")--";
		    }
		  val=sommets.at(sommets.count()-1).split(";");
		  texcode+="("+val.at(0)+","+val.at(1)+","+val.at(2)+"),"+couleur+"+dashed);\n";
		  }
	      }
	  }
      }
    }
  }
texcode+=QString::fromUtf8("// code supplémentaire\n")+ui.textEditSupp->toPlainText()+"\n"; 
texcode+="shipout(bbox(0.1cm,0.1cm,white));\n";
return texcode;
}

void EspaceDialog::AfficherCode()
{
ui.textEditCode->clear();
QString texcode=GenererCode()+"\n";
ui.textEditCode->setPlainText(texcode);
}
//*********************************************************************
void EspaceDialog::ajouterObjet()
{
ActiverObjets(true);
expressionList.append("Nouveau");
couleurList.append(ui.comboBoxCouleur->currentText());
styleList.append(ui.comboBoxStyle->currentText());
typeList.append(ui.comboBoxType->currentText());
remplissageList.append(ui.comboBoxRemplissage->currentText());
QListWidgetItem *nouveauItem = new QListWidgetItem(ui.listWidgetObjets);
nouveauItem->setText("Nouveau");
nouveauItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
ui.listWidgetObjets->setCurrentItem(nouveauItem);
ui.listWidgetObjets->currentItem()->setSelected(true);
ui.lineEditObjet->setFocus();
NettoyerCode();
}

void EspaceDialog::supprimerObjet()
{
int courant=numeroItem();
if (courant<0) return;
delete ui.listWidgetObjets->currentItem();
expressionList.removeAt(courant);
couleurList.removeAt(courant);
styleList.removeAt(courant);
typeList.removeAt(courant);
remplissageList.removeAt(courant);
ui.listWidgetObjets->setCurrentItem(ui.listWidgetObjets->item(courant));
ui.listWidgetObjets->currentItem()->setSelected(true);
actualiserItemInfo();
NettoyerCode();
}

void EspaceDialog::remonterObjet()
{
int courant=numeroItem();
if (courant<=0) return;
QListWidgetItem *item=ui.listWidgetObjets->item(courant)->clone();
delete ui.listWidgetObjets->item(courant);
ui.listWidgetObjets->insertItem(courant-1,item);
#if QT_VERSION < QT_VERSION_CHECK(5,13,0)
expressionList.swap(courant,courant-1);
couleurList.swap(courant,courant-1);
styleList.swap(courant,courant-1);
typeList.swap(courant,courant-1);
remplissageList.swap(courant,courant-1);
#else
expressionList.swapItemsAt(courant,courant-1);
couleurList.swapItemsAt(courant,courant-1);
styleList.swapItemsAt(courant,courant-1);
typeList.swapItemsAt(courant,courant-1);
remplissageList.swapItemsAt(courant,courant-1);
#endif
ui.listWidgetObjets->setCurrentItem(ui.listWidgetObjets->item(courant-1));
ui.listWidgetObjets->currentItem()->setSelected(true);
NettoyerCode();
}

void EspaceDialog::descendreObjet()
{
int courant=numeroItem();
if (courant>=ui.listWidgetObjets->count()-1 || courant<0) return;
QListWidgetItem *item=ui.listWidgetObjets->item(courant)->clone();
delete ui.listWidgetObjets->item(courant);
ui.listWidgetObjets->insertItem(courant+1,item);
#if QT_VERSION < QT_VERSION_CHECK(5,13,0)
expressionList.swap(courant,courant+1);
couleurList.swap(courant,courant+1);
styleList.swap(courant,courant+1);
typeList.swap(courant,courant+1);
remplissageList.swap(courant,courant+1);
#else
expressionList.swapItemsAt(courant,courant+1);
couleurList.swapItemsAt(courant,courant+1);
styleList.swapItemsAt(courant,courant+1);
typeList.swapItemsAt(courant,courant+1);
remplissageList.swapItemsAt(courant,courant+1);
#endif


ui.listWidgetObjets->setCurrentItem(ui.listWidgetObjets->item(courant+1));
ui.listWidgetObjets->currentItem()->setSelected(true);
NettoyerCode();
}

void EspaceDialog::actualiserExpression()
{
int courant=numeroItem();
if (courant<0) return;
ui.listWidgetObjets->item(courant)->setText(ui.lineEditObjet->text());
expressionList[courant]=ui.lineEditObjet->text();
NettoyerCode();
}

void EspaceDialog::actualiserCouleur()
{
int courant=numeroItem();
if (courant<0) return;
couleurList[courant]=ui.comboBoxCouleur->currentText();
NettoyerCode();
}

void EspaceDialog::actualiserStyle()
{
int courant=numeroItem();
if (courant<0) return;
styleList[courant]=ui.comboBoxStyle->currentText();
NettoyerCode();
}

void EspaceDialog::actualiserType()
{
int courant=numeroItem();
if (courant<0) return;
typeList[courant]=ui.comboBoxType->currentText();
NettoyerCode();
}

void EspaceDialog::actualiserRemplissage()
{
int courant=numeroItem();
if (courant<0) return;
remplissageList[courant]=ui.comboBoxRemplissage->currentText();
NettoyerCode();
}

void EspaceDialog::actualiserItemInfo()
{
int courant=numeroItem();
if (courant<0) 
	{
	ui.lineEditObjet->setText("");
	ui.comboBoxCouleur->setCurrentIndex(0);
	ui.comboBoxStyle->setCurrentIndex(0);
	ui.comboBoxType->setCurrentIndex(0);
	ui.comboBoxRemplissage->setCurrentIndex(0);
	ActiverObjets(false);
	}
else
	{
	ui.lineEditObjet->disconnect(SIGNAL(textChanged(const QString &)));
	ui.comboBoxCouleur->disconnect(SIGNAL(currentTextChanged(const QString &)));
	ui.comboBoxStyle->disconnect(SIGNAL(currentTextChanged(const QString &)));
	ui.comboBoxType->disconnect(SIGNAL(currentTextChanged(const QString &)));
	ui.comboBoxRemplissage->disconnect(SIGNAL(currentTextChanged(const QString &)));

	ui.lineEditObjet->setText(expressionList[courant]);
	ui.comboBoxCouleur->setCurrentIndex(ui.comboBoxCouleur->findText(couleurList[courant],Qt::MatchExactly));
	ui.comboBoxStyle->setCurrentIndex(ui.comboBoxStyle->findText(styleList[courant],Qt::MatchExactly));
	ui.comboBoxType->setCurrentIndex(ui.comboBoxType->findText(typeList[courant],Qt::MatchExactly));
	ui.comboBoxRemplissage->setCurrentIndex(ui.comboBoxRemplissage->findText(remplissageList[courant],Qt::MatchExactly));

	connect(ui.lineEditObjet, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserExpression()));
	connect(ui.comboBoxCouleur, SIGNAL(currentTextChanged(const QString &)),this,SLOT(actualiserCouleur()));
	connect(ui.comboBoxStyle, SIGNAL(currentTextChanged(const QString &)),this,SLOT(actualiserStyle()));
	connect(ui.comboBoxType, SIGNAL(currentTextChanged(const QString &)),this,SLOT(actualiserType()));
	connect(ui.comboBoxRemplissage, SIGNAL(currentTextChanged(const QString &)),this,SLOT(actualiserRemplissage()));
	ActiverObjets(true);
	}
}

int EspaceDialog::numeroItem()
{
QList<QListWidgetItem *> items;
items = ui.listWidgetObjets->selectedItems();
if (items.count() > 0) 
	{
	ui.listWidgetObjets->setCurrentItem(items[0]);
	return ui.listWidgetObjets->row(items[0]);
	}
return -1;
}

void EspaceDialog::Translater()
{
bool ok;
int courant=numeroItem();
if (courant<0) return;
QStringList val,points;
QString nouveaucode="";

QString sx=ui.lineEditTranslateX->text();
double tx=sx.toDouble(&ok);
if (!ok) return;
QString sy=ui.lineEditTranslateY->text();
double ty=sy.toDouble(&ok);
if (!ok) return;
QString sz=ui.lineEditTranslateZ->text();
double tz=sz.toDouble(&ok);
if (!ok) return;

points=expressionList.at(courant).split(":");
if (points.count()<=0) return;
for (int i = 0; i < points.count(); i++)
	{
	val=points.at(i).split(";");
	if (val.count()>=3)
		{
		double xorig=(double) val.at(0).toDouble();
		double nouvx=xorig+tx;
		double yorig=(double) val.at(1).toDouble();
		double nouvy=yorig+ty;
		double zorig=(double) val.at(2).toDouble();
		double nouvz=zorig+tz;
		nouveaucode+=QString::number(nouvx)+";"+QString::number(nouvy)+";"+QString::number(nouvz);
		if (val.count()>3)
			{		
			for (int j = 3; j < val.count(); j++) nouveaucode+=";"+val.at(j);
			}
		}
	if (i<points.count()-1) nouveaucode+=":";
	}
expressionList[courant]=nouveaucode;
ui.lineEditObjet->setText(nouveaucode);
ui.lineEditTranslateX->setText("0");
ui.lineEditTranslateY->setText("0");
ui.lineEditTranslateZ->setText("0");
}
//******************************************************
void EspaceDialog::NouvelEspace()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Espace : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant d'en créer un nouveau?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverEspace();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Espace : ")+nomFichier);
}

void EspaceDialog::SauverEspace()
{
QDomDocument doc;
QDomElement description;
QDomText contenu;
QDomProcessingInstruction instr =  doc.createProcessingInstruction("xml","version=\"1.0\" encoding=\"UTF-8\"");
doc.appendChild(instr);
QDomElement root=doc.createElement("Espace");
doc.appendChild(root);

description=doc.createElement("Unit");
contenu=doc.createTextNode(ui.lineEditUnite->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Coeff");
contenu=doc.createTextNode(ui.lineEditCoeff->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Xmin");
contenu=doc.createTextNode(QString::number(ui.spinBoxXmin->value()));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Xmax");
contenu=doc.createTextNode(QString::number(ui.spinBoxXmax->value()));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Ymin");
contenu=doc.createTextNode(QString::number(ui.spinBoxYmin->value()));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Ymax");
contenu=doc.createTextNode(QString::number(ui.spinBoxYmax->value()));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Zmin");
contenu=doc.createTextNode(QString::number(ui.spinBoxZmin->value()));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Zmax");
contenu=doc.createTextNode(QString::number(ui.spinBoxZmax->value()));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Labels");
if (ui.checkBoxLabels->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Supplement");
contenu=doc.createTextNode(ui.textEditSupp->toPlainText());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Objets");
contenu=doc.createTextNode(expressionList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Couleurs");
contenu=doc.createTextNode(couleurList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Styles");
contenu=doc.createTextNode(styleList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Types");
contenu=doc.createTextNode(typeList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Remplissages");
contenu=doc.createTextNode(remplissageList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

QString nouveauFichier;
if (nomFichier=="sanstitre" ) 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Espace (*.pag)"));
	}
else 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),nomFichier,QString::fromUtf8("Espace (*.pag)"));
	}
//nouveauFichier=nomFichier;
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::WriteOnly))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible d'enregistrer le fichier"));
	return;
	}
nomFichier=nouveauFichier;
QTextStream out (&fichier);
doc.save(out,4);
fichier.close();
QFileInfo fi(nomFichier);
dernierRepertoire=fi.absolutePath();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Espace : ")+nomFichier);
}

void EspaceDialog::Ouvrir(QString nouveauFichier)
{
QDomDocument doc;
QDomNodeList nodelist;
QDomNode node;
QFileInfo fi(nouveauFichier);
if (!fi.exists()) return;
dernierRepertoire=fi.absolutePath();
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::ReadOnly)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	return;
	}
if (!doc.setContent(&fichier)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	fichier.close();
	return;
	}

QDomElement root = doc.documentElement();
if (root.tagName() != "Espace")
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Ce fichier ne correspond pas à un repère dans l'espace"));
	fichier.close();
	return;
	}
nomFichier=nouveauFichier;
Init();

nodelist = doc.elementsByTagName("Unit");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditUnite->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Coeff");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditCoeff->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Xmin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.spinBoxXmin->setValue(node.toElement().text().toInt());
	}

nodelist = doc.elementsByTagName("Xmax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.spinBoxXmax->setValue(node.toElement().text().toInt());
	}

nodelist = doc.elementsByTagName("Ymin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.spinBoxYmin->setValue(node.toElement().text().toInt());
	}

nodelist = doc.elementsByTagName("Ymax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.spinBoxYmax->setValue(node.toElement().text().toInt());
	}

nodelist = doc.elementsByTagName("Zmin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.spinBoxZmin->setValue(node.toElement().text().toInt());
	}

nodelist = doc.elementsByTagName("Zmax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.spinBoxZmax->setValue(node.toElement().text().toInt());
	}

nodelist = doc.elementsByTagName("Labels");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxLabels->setChecked(true);
	else ui.checkBoxLabels->setChecked(false);
	}

nodelist = doc.elementsByTagName("Supplement");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.textEditSupp->setPlainText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Objets");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	expressionList=node.toElement().text().split("#");
	}

nodelist = doc.elementsByTagName("Couleurs");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	couleurList=node.toElement().text().split("#");
	}

nodelist = doc.elementsByTagName("Styles");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	styleList=node.toElement().text().split("#");
	}

nodelist = doc.elementsByTagName("Types");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	typeList=node.toElement().text().split("#");
	}

nodelist = doc.elementsByTagName("Remplissages");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	remplissageList=node.toElement().text().split("#");
	}

fichier.close();
ui.listWidgetObjets->clear();
for (int i = 0; i < expressionList.count(); i++)
	{
	if (!expressionList.at(i).isEmpty() && !couleurList.at(i).isEmpty() && !styleList.at(i).isEmpty() && !typeList.at(i).isEmpty() && !remplissageList.at(i).isEmpty())
		{
		QListWidgetItem *nouveauItem = new QListWidgetItem(ui.listWidgetObjets);
		nouveauItem->setText(expressionList.at(i));
		nouveauItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		}
	else
		{
		expressionList.removeAt(i);
		couleurList.removeAt(i);
		styleList.removeAt(i);
		typeList.removeAt(i);
		remplissageList.removeAt(i);
		}
	}
if (ui.listWidgetObjets->count()>0)
	{
	ui.listWidgetObjets->setCurrentItem(ui.listWidgetObjets->item(0));
    ui.listWidgetObjets->currentItem()->setSelected(true);
	}
actualiserItemInfo();
AfficherCode();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Espace : ")+nomFichier);
}

void EspaceDialog::ChargerEspace()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Espace : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de charger un autre graphe?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverEspace();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QString nouveauFichier = QFileDialog::getOpenFileName(this,QString::fromUtf8("Ouvrir Fichier"),dernierRepertoire,"Espace (*.pag)");
if (nouveauFichier.isEmpty()) return;
Ouvrir(nouveauFichier);
}
//*********************************************

void EspaceDialog::NettoyerCode()
{
ui.textEditCode->clear();
estModifie=true;
setWindowTitle(QString::fromUtf8("Assistant Espace ")+QString::fromUtf8("[modifié]")+" : "+nomFichier);
}

//*******************************
void EspaceDialog::Aide()
{
AideDialog *aideDialog = new AideDialog(this,"espace");
aideDialog->exec();
}
//*****************************
void EspaceDialog::ExporterPdf()
{
this->disconnect(SIGNAL(asy_pdf_fini()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(ExporterPdfSuite()));
AsyPdfTemp();
}

void EspaceDialog::ExporterPdfSuite()
{
if (!erreurProcessus)
	{
	QString pdfFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Pdf (*.pdf)"));
	if (!pdfFichier.isEmpty())
		{
		QFileInfo fi(pdfFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+pdfTempFile,pdfFichier);
		}
	}
this->disconnect(SIGNAL(asy_pdf_fini()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(VoirTemp()));
}

void EspaceDialog::CopierFichier(QString origine,QString destination)
{
if (destination.isEmpty() || origine.isEmpty()) return;
QFileInfo fi_or(origine);
if (!fi_or.exists()) return;
QFile fichier_or(origine);
QFileInfo fi_dest(destination);
if (fi_dest.exists())
	{
	QFile fichier_dest(destination);
	fichier_dest.remove();
	fichier_or.copy(destination);
	}
else
	{
	fichier_or.copy(destination);
	}
}

void EspaceDialog::AsyPdfTemp()
{
AfficherCode();
if (ui.textEditCode->toPlainText().contains(QRegularExpression("Erreur|erreur")))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Erreurs dans le code généré.\nLa compilation n'est pas lancée."));
	return;
	}

QFile fichier;
fichier.setFileName(tempDir+"/"+asyTempFile);
fichier.open(QIODevice::WriteOnly);
QTextCodec *codec = QTextCodec::codecForName("UTF-8");
fichier.write(codec->fromUnicode(GenererCode()));
fichier.close();

erreurProcessus=false;
asy_pdf_proc = new CmdLauncher( this, tempDir, extraPath );
connect(asy_pdf_proc, SIGNAL(jobdone()),this, SIGNAL(asy_pdf_fini()));
connect(asy_pdf_proc, SIGNAL(joberror()),this, SLOT(ErreurCompilation()));
asy_pdf_proc->launchCommand(asyCommande+" -epsdriver=ps2write -f pdf -noView "+asyTempFile);
}

void EspaceDialog::ErreurCompilation()
{
erreurProcessus=true;
}



void EspaceDialog::VoirTemp()
{
if (erreurProcessus) return;
QString fileImage=tempDir+"/"+pdfTempFile;
QFileInfo fi(fileImage);
if (fi.exists()) 
	{
	if (pdfviewerWindow)
	  {
	  pdfviewerWindow->openFile(fileImage,lecteurpdfCommande);
	  }
	else
	  {
	  pdfviewerWindow=new PdfViewer(fileImage,lecteurpdfCommande,this);
	  }  
	  pdfviewerWindow->raise();
	  pdfviewerWindow->show();
	}
}
