/***************************************************************************
 *   copyright       : (C) 2012-2021 by Pascal Brachet                     *
 *   https://www.xm1math.net/pdfadd/                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   @license GPL-2.0+ <https://spdx.org/licenses/GPL-2.0+.html>           *
 ***************************************************************************/


#include <QtGui>
#include <QRegularExpression>

#include "eukhighlighter.h"
#include "blockdata.h"

EukHighlighter::EukHighlighter(QTextDocument *parent)
    : QSyntaxHighlighter(parent)
{
ColorStandard = QColor("#000000");
ColorCommande=QColor("#800000");
CommandeWords= QString("\\bdraw\\b,\\bmark\\b,\\bpoint\\b,\\bbarycenter\\b,\\bmidpoint\\b,\\borthocenter\\b,\\bprojection\\b,\\btranslation\\b,\\breflection\\b,\\brotation\\b,\\bcenter\\b,\\bbegin\\b,\\bend\\b,\\babscissa\\b,\\bordinate\\b,\\bline\\b,\\bparallel\\b,\\bperpendicular\\b,\\bbisector\\b,\\baltitude\\b,\\bmedian\\b,\\bhomothecy\\b,\\bsegment\\b,\\bcircle\\b,\\bincircle\\b,\\bintersection\\b,\\btriangle\\b,\\bright\\b,\\bisosceles\\b,\\bequilateral\\b,\\bparallelogram\\b,\\brectangle\\b,\\bsquare\\b,\\bvector\\b").split(",");
}

EukHighlighter::~EukHighlighter(){
}

void EukHighlighter::highlightBlock(const QString &text)
{
int i = 0;
int state = previousBlockState();
if (state<0) state=0;
QChar last, next ,ch,tmp;
QString buffer;
const int StateStandard = 0;
const int StateComment = 1;
const int StateString = 2;

BlockData *blockData = new BlockData;
int leftPos = text.indexOf( '(' );
while ( leftPos != -1 ) 
  {
  ParenthesisInfo *info = new ParenthesisInfo;
  info->character = '(';
  info->position = leftPos;

  blockData->insertPar( info );
  leftPos = text.indexOf( '(', leftPos+1 );
  }

int rightPos = text.indexOf(')');
while ( rightPos != -1 ) 
  {
  ParenthesisInfo *info = new ParenthesisInfo;
  info->character = ')';
  info->position = rightPos;

  blockData->insertPar( info );
  rightPos = text.indexOf( ')', rightPos+1 );
  }
setCurrentBlockUserData(blockData);
blockData->code.clear(); 

for (int j=0; j < text.length(); j++) blockData->code.append(0);
while (i < text.length())
    {
    ch = text.at( i );
    buffer += ch;
    if ( i < text.length()-1 ) next = text.at( i+1 );

    switch (state) 
    {
	case StateStandard: 
	{
	tmp=text.at( i );
	if (tmp== '(' )
	    {
	    blockData->code[i]=1;
	    setFormat( i, 1,ColorStandard);
	    state=StateStandard;
	    } 
	else if (tmp== ')' )
	    {
	    blockData->code[i]=1;
	    setFormat( i, 1,ColorStandard);
	    state=StateStandard;
	    } 
	else if (isWordSeparator(tmp))
	    {
	    blockData->code[i]=1;
	    setFormat( i, 1,ColorStandard);
	    } 
	else
	    {
	    setFormat( i, 1,ColorStandard);
	    state=StateStandard;
	    }
	buffer = QString();
	} break;
    }
    last = ch;
    i++;
    }
setCurrentBlockState(StateStandard) ;

if (text.isEmpty()) return;
i=0;
while (i < text.length())
	{
	buffer = QString();
	ch = text.at( i );
	while ((blockData->code[i]!=1) && (!isSpace(ch)))
	      {
	      buffer += ch;
	      i++;
	      if (i < text.length()) ch = text.at( i );
	      else break;
	      }
	if ( buffer.length() > 0 )
		{
		for ( QStringList::Iterator it = CommandeWords.begin(); it != CommandeWords.end(); ++it ) 
			{
			QRegularExpression expression(( *it ),QRegularExpression::CaseInsensitiveOption);
            int index=expression.match(buffer).capturedStart();
			while (index >= 0) 
			  {
            int length=expression.match(buffer).capturedLength();
			  setFormat(i-length, length,ColorCommande);
			  index = expression.match(buffer, index + length).capturedStart();
			  }
			}
		}
	i++;
	}
}

bool EukHighlighter::isWordSeparator(QChar c) const
{
    switch (c.toLatin1()) {
    case '.':
    case ',':
    case '?':
    case '!':
    case ':':
    case ';':
    case '-':
    case '<':
    case '>':
    case '[':
    case ']':
    case '(':
    case ')':
    case '{':
    case '}':
    case '=':
    case '/':
    case '+':
    case '%':
    case '&':
    case '^':
    case '*':
    case '\'':
    case '"':
    case '~':
        return true;
    default:
        return false;
    }
}

bool EukHighlighter::isSpace(QChar c) const
{
    return c == QLatin1Char(' ')
        || c == QChar::Nbsp
        || c == QChar::LineSeparator
        || c == QLatin1Char('\t')
        ;
}

