/***************************************************************************
 *   copyright       : (C) 2012-2021 by Pascal Brachet                     *
 *   https://www.xm1math.net/pdfadd/                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   @license GPL-2.0+ <https://spdx.org/licenses/GPL-2.0+.html>           *
 ***************************************************************************/

#ifndef GEOMETRIEDIALOG_H
#define GEOMETRIEDIALOG_H

#include "ui_geometriedialog.h"
#include "highlighter.h"
#include "pdfviewer.h"
#include "cmdlauncher.h"

#include <QPointer>
#include <QCloseEvent>
#include <QListWidget>
#include <QListWidgetItem>
#include <QCompleter>

class GeometrieDialog : public QDialog  {
   Q_OBJECT
public:
	GeometrieDialog(QWidget *parent=0);
	~GeometrieDialog();
	Ui::GeometrieDialog ui;
QString extraPath, pdflatexCommande, asyCommande, lecteurpdfCommande, dernierRepertoire, nomFichier, preambuleAsy;
private :
QString tempDir,asyTempFile,pdfTempFile,prefixFile,eukTempFile;
QPointer<CmdLauncher> asy_pdf_proc;
bool estModifie,existvaleur, erreurProcessus;
QListWidget *dessinListWidget, *pointsListWidget, *lignesListWidget, *segmentsListWidget, *cerclesListWidget, *trianglesListWidget, *polygonesListWidget, *vecteursListWidget;
Highlighter *highlighterSupp;
QPointer<PdfViewer> pdfviewerWindow;
QCompleter *completer;
public slots:
void Ouvrir(QString nouveauFichier);
private slots:
void closeEvent(QCloseEvent *e);
void accept();
void reject();

void ConstruireAideCommandes();
void InsererCommande(QListWidgetItem* item);
void Init();

void ActiverAxes(bool etat);
void ActiverGrille(bool etat);

bool Operateur(QString mot);
bool Fonction(QString mot);
int Priorite(QString mot);
QString CalculOp(QString n,QString m,QString x,QString variable);
QString CalculAp(QString n,QString x,QString variable);
QString CalculerExpression(QString expression, QString variable);
QString CalculerExpressionArrondi(QString expression, QString variable);

QString GenererCode();
void AfficherCode();

void NouvelleFigure();
void SauverFigure();
void ChargerFigure();

void AsyPdfTemp();
void VoirTemp();
void ErreurCompilation();

void NettoyerCode();
void NettoyerSupp();
void NettoyerCommandes();


void Aide();

void ExporterPdf();
void ExporterPdfSuite();
void CopierFichier(QString origine,QString destination);
signals:
void asy_pdf_fini();
};

#endif
