/***************************************************************************
 *   copyright       : (C) 2012-2021 by Pascal Brachet                     *
 *   https://www.xm1math.net/pdfadd/                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   @license GPL-2.0+ <https://spdx.org/licenses/GPL-2.0+.html>           *
 ***************************************************************************/

#include "histodialog.h"
#include "aidedialog.h"
#include "logdialog.h"

#include <QDebug>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QDomNodeList>
#include <QDomProcessingInstruction>
#include <QFile>
#include <QDir>
#include <QMessageBox>
#include <QFileDialog>
#include <QFileInfo>
#include <QUrl>
#include <QFontDatabase>
#include <QRegularExpression>
#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
    #include <QtCore/QTextCodec>
#else
    #include <QtCore5Compat/QTextCodec>
#endif


#include <math.h>

HistoDialog::HistoDialog(QWidget *parent)
    :QDialog( parent)
{
ui.setupUi(this);
#if defined(Q_OS_MAC)
setModal(true);
#endif
connect(ui.barreOutils->actionQuitter, SIGNAL(triggered()), this, SLOT(accept()));
connect(ui.barreOutils->actionGenerer, SIGNAL(triggered()), this, SLOT(AfficherCode()));
connect(ui.barreOutils->actionNouveau, SIGNAL(triggered()), this, SLOT(NouvelHistogramme()));
connect(ui.barreOutils->actionOuvrir, SIGNAL(triggered()), this, SLOT(ChargerHistogramme()));
connect(ui.barreOutils->actionSauver, SIGNAL(triggered()), this, SLOT(SauverHistogramme()));
connect(ui.barreOutils->actionAide, SIGNAL(triggered()), this, SLOT(Aide()));
connect(ui.barreOutils->actionExporterPdf, SIGNAL(triggered()), this, SLOT(ExporterPdf()));
connect(ui.barreOutils->actionApercu, SIGNAL(triggered()), this, SLOT(AsyPdfTemp()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(VoirTemp()));

connect(ui.listWidgetRectangles, SIGNAL(itemSelectionChanged()),this, SLOT(actualiserItemInfo()));
connect(ui.lineEditRectangle, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserExpression()));
connect(ui.comboBoxCouleur, SIGNAL(currentTextChanged(const QString &)),this,SLOT(actualiserCouleur()));
connect(ui.comboBoxStyle, SIGNAL(currentTextChanged(const QString &)),this,SLOT(actualiserStyle()));
connect(ui.ajouterButton, SIGNAL(clicked()), this, SLOT(ajouterRectangle()));
connect(ui.supprimerButton, SIGNAL(clicked()), this, SLOT(supprimerRectangle()));
connect(ui.checkBoxAxes, SIGNAL(toggled(bool)),this, SLOT(ActiverAxes(bool)));
connect(ui.checkBoxGrille, SIGNAL(toggled(bool)),this, SLOT(ActiverGrille(bool)));

connect(ui.lineEditUniteX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditXmin, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditXmax, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGrilleX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGradX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));

connect(ui.lineEditUniteY, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditYmin, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditYmax, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGrilleY, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGradY, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));

connect(ui.textEditSupp, SIGNAL(textChanged()),this,SLOT(NettoyerCode()));

connect(ui.checkBoxLabels, SIGNAL(toggled(bool)),this, SLOT(NettoyerCode()));





ui.listWidgetRectangles->setAlternatingRowColors(true);
ui.listWidgetRectangles->setSelectionMode (QAbstractItemView::SingleSelection);

ui.comboBoxCouleur->addItems(QString("couleur1,couleur2,couleur3").split(","));

tempDir=QDir::tempPath();
prefixFile=QDir::homePath();
prefixFile="pdfadd_temp_"+prefixFile.section('/',-1);
prefixFile=QString(QUrl::toPercentEncoding(prefixFile));
prefixFile.remove("%");
asyTempFile=prefixFile+".asy";
pdfTempFile=prefixFile+".pdf";

highlighterSupp = new Highlighter(ui.textEditSupp->document());

QFont efont;
QFontDatabase fdb;
QStringList xf = fdb.families();
QString deft;
if (xf.contains("Liberation Mono",Qt::CaseInsensitive)) deft="Liberation Mono";
else if (xf.contains("DejaVu Sans Mono",Qt::CaseInsensitive)) deft="DejaVu Sans Mono";
#if defined(Q_OS_MAC)
else if (xf.contains("Courier",Qt::CaseInsensitive)) deft="Courier";
#endif
#if defined(Q_OS_WIN32)
else if (xf.contains("Courier New",Qt::CaseInsensitive)) deft="Courier New";
#endif
else deft=qApp->font().family();  
efont.setFamily(deft);
efont.setPointSize(qApp->font().pointSize());
ui.textEditCode->setFont(efont);
ui.textEditSupp->setFont(efont);
ui.textEditCode->setWordWrapMode(QTextOption::NoWrap);
ui.textEditSupp->setWordWrapMode(QTextOption::NoWrap);
ui.lineEditRectangle->setFont(efont);
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Histogramme : ")+nomFichier);
show();
}

HistoDialog::~HistoDialog(){
// if (asy_pdf_proc && asy_pdf_proc->state()==QProcess::Running)
// 	{
// 	asy_pdf_proc->disconnect();
// 	asy_pdf_proc->kill();
// 	delete asy_pdf_proc ;
// 	}
}

void HistoDialog::closeEvent(QCloseEvent *e)
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Histogramme : ",
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverHistogramme();
		break;
		case 1:
		break;
		case 2:
		default:
		e->ignore();
		return;
		break;
		}
	}
e->accept();
}

void HistoDialog::accept()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Histogramme : ",
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverHistogramme();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::accept();
}

void HistoDialog::reject()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Histogramme : ",
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverHistogramme();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::reject();
}

void HistoDialog::Init()
{
ui.textEditCode->clear();
expressionList.clear();
couleurList.clear();
styleList.clear();
ui.listWidgetRectangles->clear();
ui.checkBoxAxes->setChecked(true);
ui.checkBoxLabels->setChecked(true);
ui.checkBoxGrille->setChecked(false);
ActiverGrille(false);
ui.lineEditUniteX->setText("1");
ui.lineEditXmin->setText("0");
ui.lineEditXmax->setText("1");
ui.lineEditGradX->setText("1");
ui.lineEditGrilleX->setText("1");
ui.lineEditUniteY->setText("1");
ui.lineEditYmin->setText("0");
ui.lineEditYmax->setText("1");
ui.lineEditGradY->setText("1");
ui.lineEditGrilleY->setText("1");
ui.textEditSupp->clear();
ActiverRectangles(false);
ActiverGrille(false);
}
//*********************************************
void HistoDialog::ActiverRectangles(bool etat)
{
ui.lineEditRectangle->setEnabled(etat);
ui.comboBoxCouleur->setEnabled(etat);
ui.comboBoxStyle->setEnabled(etat);
//NettoyerCode();
}
void HistoDialog::ActiverAxes(bool etat)
{
ui.lineEditGradX->setEnabled(etat);
ui.lineEditGradY->setEnabled(etat);
ui.checkBoxLabels->setEnabled(etat);
NettoyerCode();
}
void HistoDialog::ActiverGrille(bool etat)
{
ui.lineEditGrilleX->setEnabled(etat);
ui.lineEditGrilleY->setEnabled(etat);
NettoyerCode();
}
//******************************************************************
bool HistoDialog::Operateur(QString mot)
{
bool result=false;
if ( (mot=="(") || (mot==")") || (mot=="-") || (mot=="+") || (mot=="*") || (mot=="/") || (mot=="^") ) result=true;
return result;
}

bool HistoDialog::Fonction(QString mot)
{
bool result=false;
if ( (mot=="cos") || (mot=="sin") || (mot=="sqrt") || (mot=="ln") || (mot=="log") || (mot=="exp") || (mot=="tan") || (mot=="abs") || (mot=="acos") || (mot=="asin") || (mot=="atan") ) result=true;
return result;
}

int HistoDialog::Priorite(QString mot)
{
int result=-1;
if (mot=="(") result=0;
else if (mot==")") result=1;
else if (mot=="-") result=2;
else if (mot=="+") result=2;
else if (mot=="*") result=3;
else if (mot=="/") result=3;
else if (mot=="^") result=5;
else if (Fonction(mot)) result=4;
return result;
}

QString HistoDialog::CalculOp(QString n,QString m,QString x,QString variable)
{
double calcul=0;
double p,q;
bool ok;
if (n=="x") p=variable.toDouble(&ok);
else p=n.toDouble(&ok);
if (m=="x") q=variable.toDouble(&ok);
else q=m.toDouble(&ok);
if (x=="+") calcul=q+p;
if (x=="-") calcul=q-p;
if (x=="*") calcul=q*p;
if (x=="/") 
	{
	if (p!=0) calcul=q/p;
	else existvaleur=false;
	}
if (x=="^") 
{
	if ((floor(abs(p))==(abs(p))) && (p>=0)) 
		{
		if (p==0) calcul=1;
		if (p==1) calcul=q;
		if (p>1) 
			{
			calcul=1;
			for (int i = 1; i <=p; i++) calcul=calcul*q;
			}
		}
	else 
		{
		if (q>1e-4) calcul=exp(p*log(abs(q)));
		else existvaleur=false;
		}
}
return QString::number(calcul,'g',12);
}

QString HistoDialog::CalculAp(QString n,QString x,QString variable)
{
double calcul=0;
double p;
bool ok;
if (n=="x") p=variable.toDouble(&ok);
else p=n.toDouble(&ok);
if (x=="cos")  calcul=cos(p);
if (x=="sin")  calcul=sin(p);
if (x=="abs")  calcul=abs(p);
if (x=="sqrt") 
   {
   if (p>=0)  calcul=sqrt(p);
   else existvaleur=false;
   }
if (x=="ln") 
   {
   if (p>1e-50)  calcul=log(p);
   else existvaleur=false;
   }
if (x=="exp") 
   {
   if ((p<80) && (p>-80))  calcul=exp(p);
   else existvaleur=false;
   }
if (x=="tan") 
   {
   if (cos(p)!=0) calcul=tan(p);
   else existvaleur=false;
   }
if (x=="acos") 
   {
   if ((p<=1) && (p>=-1)) calcul=acos(p);
   else existvaleur=false;
   }
if (x=="asin") 
   {
   if ((p<=1) && (p>=-1)) calcul=asin(p);
   else existvaleur=false;
   }
if (x=="atan") calcul=atan(p);
if (x=="log") 
   {
   if (p>1e-50)  calcul=log10(p);
   else existvaleur=false;
   }
return QString::number(calcul,'g',12);
}


QString HistoDialog::CalculerExpression(QString expression, QString variable)
{
QStringList Pp,Rp,Ep,Cp;
Pp.clear();
Rp.clear();
Ep.clear();
Cp.clear();
QString lettre,mot,n,m,z,result;
//**** PARSEUR et CREATION PILES
int i;
bool fini;
if (expression.isEmpty()) return result;
expression+="f";
i=0;
fini=false;
do
	{
	lettre=expression.mid(i,1);
	i+=1;
	if (lettre=="-")
		{
		if (Ep.isEmpty()) Ep.append("0");
		else if (Ep.last()=="(") Ep.append("0");
		}
	if (Operateur(lettre) || lettre=="x") Ep.append(lettre);
	else
		{
		mot="";
		while ((!Operateur(lettre)) && (lettre!="f")) 
			{
			mot+=lettre;
			lettre=expression.mid(i,1);
			i+=1;
			}
		if (mot=="pi") mot="3.141592653589793238";
		if (!mot.isEmpty()) Ep.append(mot);
		if (lettre=="f") 
			{
			fini=true;
			Ep.append("f");
			}
		else Ep.append(lettre);
		}
	}
while (!fini);
//******* POSTFIXE
i=0;
mot="";
Pp.append("(");
mot=Ep.at(i);
while (mot!="f") 
	{
	if ((!Operateur(mot)) && (!Fonction(mot))) Rp.append(mot);
	else
		{
		if (mot=="(") Pp.append(mot);
		else
			{
			while (Priorite(Pp.last())>=Priorite(mot)) 
				{
				Rp.append(Pp.last());
				Pp.removeLast();
				}
			if (mot==")")  Pp.removeLast();
			if (mot!=")")  Pp.append(mot);
			}
		}
	i++;
	mot=Ep.at(i);
	}
while (Pp.last()!="(" && !Pp.isEmpty())
      {
      Rp.append(Pp.last());
      Pp.removeLast();
      }
if (Rp.isEmpty()) Rp.append("0");
//******* CALCULER
existvaleur=true;
for (int ind=0;ind<Rp.count();ind++) 
	{
	mot=Rp.at(ind);
	if (Operateur(mot)) 
		{
		n=Cp.last();
		Cp.removeLast();
		m=Cp.last();
		Cp.removeLast();
		z=CalculOp(n,m,mot,variable);
		Cp.append(z);
		}
	else
	{
	if (Fonction(mot))
		{
		n=Cp.last();
		Cp.removeLast();
		z=CalculAp(n,mot,variable);
		Cp.append(z);
		}
	else 
		{
		Cp.append(mot);
		}
	}
	}
if (Cp.first()=="x") result=variable;
else result=Cp.first();
if (!existvaleur) result="Erreur";
return result;
}

QString HistoDialog::CalculerExpressionArrondi(QString expression, QString variable)
{
QString result;
double num_result, num_resultbis;
int approx_result;
bool ok;
result=CalculerExpression(expression,variable);
if (result!="Erreur") 
	{
	num_result=result.toDouble(&ok);
	if (ok) 
		{
		num_resultbis=floor (num_result*1e5);
		approx_result=(int) num_resultbis;
		num_result=(double) (approx_result/1e5);
		result=QString::number(num_result,'g',12);
		}
	}
return result;
}


QString HistoDialog::GenererCode()
{
QString couleur,style,s_min,s_max,s_hauteur;
QStringList val;
bool ok;

QString s_unitX=CalculerExpressionArrondi(ui.lineEditUniteX->text(),"1");
double unitX;
if (s_unitX!="Erreur") unitX=s_unitX.toDouble(&ok);
else unitX=1;

QString s_unitY=CalculerExpressionArrondi(ui.lineEditUniteY->text(),"1");
double unitY;
if (s_unitY!="Erreur") unitY=s_unitY.toDouble(&ok);
else unitY=1;

QString s_Xmin=CalculerExpressionArrondi(ui.lineEditXmin->text(),"1");
double Xmin;
if (s_Xmin!="Erreur") Xmin=s_Xmin.toDouble(&ok);
else Xmin=0;

QString s_Ymin=CalculerExpressionArrondi(ui.lineEditYmin->text(),"1");
double Ymin;
if (s_Ymin!="Erreur") Ymin=s_Ymin.toDouble(&ok);
else Ymin=0;

QString s_Xmax=CalculerExpressionArrondi(ui.lineEditXmax->text(),"1");
double Xmax;
if (s_Xmax!="Erreur") Xmax=s_Xmax.toDouble(&ok);
else Xmax=1;

QString s_Ymax=CalculerExpressionArrondi(ui.lineEditYmax->text(),"1");
double Ymax;
if (s_Ymax!="Erreur") Ymax=s_Ymax.toDouble(&ok);
else Ymax=1;

QString s_gradX=CalculerExpressionArrondi(ui.lineEditGradX->text(),"1");
double gradX;
if (s_gradX!="Erreur") gradX=s_gradX.toDouble(&ok);
else gradX=1;

QString s_gradY=CalculerExpressionArrondi(ui.lineEditGradY->text(),"1");
double gradY;
if (s_gradY!="Erreur") gradY=s_gradY.toDouble(&ok);
else gradY=1;

QString s_grilleX=CalculerExpressionArrondi(ui.lineEditGrilleX->text(),"1");
double grilleX;
if (s_grilleX!="Erreur") grilleX=s_grilleX.toDouble(&ok);
else grilleX=1;
if (grilleX<=0) grilleX=1;

QString s_grilleY=CalculerExpressionArrondi(ui.lineEditGrilleY->text(),"1");
double grilleY;
if (s_grilleY!="Erreur") grilleY=s_grilleY.toDouble(&ok);
else grilleY=1;
if (grilleY<=0) grilleY=1;

double grilleXmin=Xmin/grilleX;
double grilleXmax=Xmax/grilleX;
double grilleYmin=Ymin/grilleY;
double grilleYmax=Ymax/grilleY;
double grilleUnitX=grilleX*unitX;
double grilleUnitY=grilleY*unitY;
QString s_Ox="0";
QString s_Oy="0";
if (Xmin*Xmax>0) s_Ox=s_Xmin;
if (Ymin*Ymax>0) s_Oy=s_Ymin;
QString texcode=QString::fromUtf8("// préambule asymptote\n")+preambuleAsy+"\n// code figure\n";
texcode+="import graph;\n";
texcode+="import patterns;\n";
texcode+="unitsize("+s_unitX+"cm,"+s_unitY+"cm);\n";
texcode+="xlimits("+s_Xmin+","+s_Xmax+");\n";
texcode+="ylimits("+s_Ymin+","+s_Ymax+");\n";



int pR1= (couleurSupp1.red()*10000/255);
double cR1=(double) pR1/10000;
int pG1= (couleurSupp1.green()*10000/255);
double cG1=(double) pG1/10000;
int pB1= (couleurSupp1.blue()*10000/255);
double cB1=(double) pB1/10000;
QString color1="rgb("+QString::number(cR1 ,'g',5)+","+QString::number(cG1 ,'g',5)+","+QString::number(cB1 ,'g',5)+")";

int pR2= (couleurSupp2.red()*10000/255);
double cR2=(double) pR2/10000;
int pG2= (couleurSupp2.green()*10000/255);
double cG2=(double) pG2/10000;
int pB2= (couleurSupp2.blue()*10000/255);
double cB2=(double) pB2/10000;
QString color2="rgb("+QString::number(cR2 ,'g',5)+","+QString::number(cG2 ,'g',5)+","+QString::number(cB2 ,'g',5)+")";

int pR3= (couleurSupp3.red()*10000/255);
double cR3=(double) pR3/10000;
int pG3= (couleurSupp3.green()*10000/255);
double cG3=(double) pG3/10000;
int pB3= (couleurSupp3.blue()*10000/255);
double cB3=(double) pB3/10000;
QString color3="rgb("+QString::number(cR3 ,'g',5)+","+QString::number(cG3 ,'g',5)+","+QString::number(cB3 ,'g',5)+")";


for (int i = 0; i < expressionList.count(); i++)
	{
	val=expressionList.at(i).split(";");
	if (val.count()==3)
		{
		s_min=CalculerExpressionArrondi(val.at(0),"1");
		s_max=CalculerExpressionArrondi(val.at(1),"1");
		s_hauteur=CalculerExpressionArrondi(val.at(2),"1");
		if (couleurList.at(i)=="noir") couleur="black";
		else if (couleurList.at(i)=="gris") couleur="gray";
		else if (couleurList.at(i)==QString::fromUtf8("gris foncé")) couleur="heavygray";
		else if (couleurList.at(i)=="gris clair") couleur="mediumgray";
		else if (couleurList.at(i)=="bleu") couleur="blue";
		else if (couleurList.at(i)=="rouge") couleur="red";
		else if (couleurList.at(i)=="vert") couleur="green";
		else if (couleurList.at(i)=="jaune") couleur="yellow";
		else if (couleurList.at(i)=="cyan") couleur="cyan";
		else if (couleurList.at(i)=="magenta") couleur="magenta";
		else if (couleurList.at(i)=="couleur1") couleur=color1;
		else if (couleurList.at(i)=="couleur2") couleur=color2;
		else if (couleurList.at(i)=="couleur3") couleur=color3;
		else couleur="black";
texcode+="path rectangle"+QString::number(i)+"=(("+s_min+",0)--("+s_min+","+s_hauteur+")--("+s_max+","+s_hauteur+")--("+s_max+",0)--cycle);\n";
	  if (styleList.at(i)=="plein") 
		  {
		  texcode+="filldraw(rectangle"+QString::number(i)+","+couleur+",1bp+0.7"+couleur+");\n";
		  }
	  else if (styleList.at(i)=="lignes")
		  {
		  texcode+="add(\"hachure"+QString::number(i)+"\",hatch(3mm,"+couleur+"));\n";
		  texcode+="filldraw(rectangle"+QString::number(i)+",pattern(\"hachure"+QString::number(i)+"\"),1bp+0.7"+couleur+");\n";
		  }
	  else if (styleList.at(i)=="hachures") 
		  {
		  texcode+="add(\"hachure"+QString::number(i)+"\",crosshatch(3mm,"+couleur+"));\n";
		  texcode+="filldraw(rectangle"+QString::number(i)+",pattern(\"hachure"+QString::number(i)+"\"),1bp+0.7"+couleur+");\n";
		  }
	  else if (styleList.at(i)=="vide")
		  {
		  texcode+="draw(rectangle"+QString::number(i)+",1bp+"+couleur+");\n";
		  }
	  else
		  {
		  texcode+="filldraw(rectangle"+QString::number(i)+","+couleur+",1bp+0.7"+couleur+");\n";
		  }
		}
	else
		{
		texcode+="Erreur\n";
		}
	}
if (ui.checkBoxGrille->isChecked())
  {
   texcode+="xaxis(BottomTop,xmin="+s_Xmin+",xmax="+s_Xmax+",Ticks(\"%\",extend=true,Step="+s_grilleX+"),p=linewidth(0.5pt)+gray(0.4));\n";
   texcode+="yaxis(LeftRight,ymin="+s_Ymin+",ymax="+s_Ymax+",Ticks(\"%\",extend=true,Step="+s_grilleY+"),p=linewidth(0.5pt)+gray(0.4));\n";
  }
QString ticksX="";
QString ticksY="";
//QString arrow=",Arrow(2mm)";

if (ui.checkBoxAxes->isChecked())
  {
  if (ui.checkBoxLabels->isChecked())
    {
    ticksX=",Ticks(scale(.7)*Label(),beginlabel=true,endlabel=true,begin=true,end=true,Step="+s_gradX+",Size=1mm)";
    ticksY=",Ticks(\"%\",NoZero,Step="+s_gradY+",Size=1mm)";
    //ticksY=",Ticks(scale(.7)*Label(),beginlabel=false,endlabel=false,NoZero,Step="+s_gradY+",Size=1mm)";
    }
  else
    {
    ticksX=",Ticks(\"%\",NoZero,Step="+s_gradX+",Size=1mm)";
    ticksY=",Ticks(\"%\",NoZero,Step="+s_gradY+",Size=1mm)";
    }
  texcode+="xaxis(axis=YEquals("+s_Oy+"),xmin="+s_Xmin+",xmax="+s_Xmax+ticksX+",p=linewidth(1pt)+black,true);\n";
  texcode+="yaxis(axis=XEquals("+s_Ox+"),ymin="+s_Ymin+",ymax="+s_Ymax+ticksY+",p=linewidth(1pt)+black,true);\n";
  }

texcode+=QString::fromUtf8("// code supplémentaire\n")+ui.textEditSupp->toPlainText()+"\n"; 
texcode+="shipout(bbox(0.1cm,0.1cm,white));\n";
return texcode;
}

void HistoDialog::AfficherCode()
{
ui.textEditCode->clear();
QString texcode=GenererCode()+"\n";
ui.textEditCode->setPlainText(texcode);
}
//*********************************************************************
void HistoDialog::ajouterRectangle()
{
ActiverRectangles(true);
expressionList.append("Nouveau");
couleurList.append(ui.comboBoxCouleur->currentText());
styleList.append(ui.comboBoxStyle->currentText());
QListWidgetItem *nouveauItem = new QListWidgetItem(ui.listWidgetRectangles);
nouveauItem->setText("Nouveau");
nouveauItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
ui.listWidgetRectangles->setCurrentItem(nouveauItem);
ui.listWidgetRectangles->currentItem()->setSelected(true);
ui.lineEditRectangle->setFocus();
NettoyerCode();
}

void HistoDialog::supprimerRectangle()
{
int courant=numeroItem();
if (courant<0) return;
delete ui.listWidgetRectangles->currentItem();
expressionList.removeAt(courant);
couleurList.removeAt(courant);
styleList.removeAt(courant);
ui.listWidgetRectangles->setCurrentItem(ui.listWidgetRectangles->item(courant));
ui.listWidgetRectangles->item(courant)->setSelected(true);
actualiserItemInfo();
NettoyerCode();
}

void HistoDialog::actualiserExpression()
{
int courant=numeroItem();
if (courant<0) return;
ui.listWidgetRectangles->item(courant)->setText(ui.lineEditRectangle->text());
expressionList[courant]=ui.lineEditRectangle->text();
NettoyerCode();
}

void HistoDialog::actualiserCouleur()
{
int courant=numeroItem();
if (courant<0) return;
couleurList[courant]=ui.comboBoxCouleur->currentText();
NettoyerCode();
}

void HistoDialog::actualiserStyle()
{
int courant=numeroItem();
if (courant<0) return;
styleList[courant]=ui.comboBoxStyle->currentText();
NettoyerCode();
}

void HistoDialog::actualiserItemInfo()
{
int courant=numeroItem();
if (courant<0) 
	{
	ui.lineEditRectangle->setText("");
	ui.comboBoxCouleur->setCurrentIndex(0);
	ui.comboBoxStyle->setCurrentIndex(0);
	ActiverRectangles(false);
	}
else
	{
	ui.lineEditRectangle->disconnect(SIGNAL(textChanged(const QString &)));
	ui.comboBoxCouleur->disconnect(SIGNAL(currentTextChanged(const QString &)));
	ui.comboBoxStyle->disconnect(SIGNAL(currentTextChanged(const QString &)));

	ui.lineEditRectangle->setText(expressionList[courant]);
	ui.comboBoxCouleur->setCurrentIndex(ui.comboBoxCouleur->findText(couleurList[courant],Qt::MatchExactly));
	ui.comboBoxStyle->setCurrentIndex(ui.comboBoxStyle->findText(styleList[courant],Qt::MatchExactly));

	connect(ui.lineEditRectangle, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserExpression()));
	connect(ui.comboBoxCouleur, SIGNAL(currentTextChanged(const QString &)),this,SLOT(actualiserCouleur()));
	connect(ui.comboBoxStyle, SIGNAL(currentTextChanged(const QString &)),this,SLOT(actualiserStyle()));
	ActiverRectangles(true);
	}
}

int HistoDialog::numeroItem()
{
QList<QListWidgetItem *> items;
items = ui.listWidgetRectangles->selectedItems();
if (items.count() > 0) 
	{
	ui.listWidgetRectangles->setCurrentItem(items[0]);
	return ui.listWidgetRectangles->row(items[0]);
	}
return -1;
}
//******************************************************
void HistoDialog::NouvelHistogramme()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Histogramme : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant d'en créer un nouveau?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverHistogramme();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Histogramme : ")+nomFichier);
}

void HistoDialog::SauverHistogramme()
{
QDomDocument doc;
QDomElement description;
QDomText contenu;
QDomProcessingInstruction instr =  doc.createProcessingInstruction("xml","version=\"1.0\" encoding=\"UTF-8\"");
doc.appendChild(instr);
QDomElement root=doc.createElement("Histogramme");
doc.appendChild(root);

description=doc.createElement("UnitX");
contenu=doc.createTextNode(ui.lineEditUniteX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Xmin");
contenu=doc.createTextNode(ui.lineEditXmin->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Xmax");
contenu=doc.createTextNode(ui.lineEditXmax->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("UnitY");
contenu=doc.createTextNode(ui.lineEditUniteY->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Ymin");
contenu=doc.createTextNode(ui.lineEditYmin->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Ymax");
contenu=doc.createTextNode(ui.lineEditYmax->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GradX");
contenu=doc.createTextNode(ui.lineEditGradX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GradY");
contenu=doc.createTextNode(ui.lineEditGradY->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GrilleX");
contenu=doc.createTextNode(ui.lineEditGrilleX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GrilleY");
contenu=doc.createTextNode(ui.lineEditGrilleY->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Axes");
if (ui.checkBoxAxes->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Labels");
if (ui.checkBoxLabels->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Grille");
if (ui.checkBoxGrille->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Supplement");
contenu=doc.createTextNode(ui.textEditSupp->toPlainText());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Rectangles");
contenu=doc.createTextNode(expressionList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Couleurs");
contenu=doc.createTextNode(couleurList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Styles");
contenu=doc.createTextNode(styleList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

QString nouveauFichier;
if (nomFichier=="sanstitre" ) 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Histogramme (*.pag)"));
	}
else 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),nomFichier,QString::fromUtf8("Histogramme (*.pag)"));
	}
//nouveauFichier=nomFichier;
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::WriteOnly))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible d'enregistrer le fichier"));
	return;
	}
nomFichier=nouveauFichier;
QTextStream out (&fichier);
doc.save(out,4);
fichier.close();
QFileInfo fi(nomFichier);
dernierRepertoire=fi.absolutePath();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Histogramme : ")+nomFichier);
}

void HistoDialog::Ouvrir(QString nouveauFichier)
{
QDomDocument doc;
QDomNodeList nodelist;
QDomNode node;
QFileInfo fi(nouveauFichier);
if (!fi.exists()) return;
dernierRepertoire=fi.absolutePath();
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::ReadOnly)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	return;
	}
if (!doc.setContent(&fichier)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	fichier.close();
	return;
	}

QDomElement root = doc.documentElement();
if (root.tagName() != "Histogramme")
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Ce fichier ne correspond pas à un histogramme"));
	fichier.close();
	return;
	}
nomFichier=nouveauFichier;
Init();

nodelist = doc.elementsByTagName("UnitX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditUniteX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Xmin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditXmin->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Xmax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditXmax->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("UnitY");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditUniteY->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Ymin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditYmin->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Ymax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditYmax->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GradX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGradX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GradY");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGradY->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GrilleX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGrilleX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GrilleY");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGrilleY->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Axes");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxAxes->setChecked(true);
	else ui.checkBoxAxes->setChecked(false);
	}

nodelist = doc.elementsByTagName("Labels");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxLabels->setChecked(true);
	else ui.checkBoxLabels->setChecked(false);
	}

nodelist = doc.elementsByTagName("Grille");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxGrille->setChecked(true);
	else ui.checkBoxGrille->setChecked(false);
	}

nodelist = doc.elementsByTagName("Supplement");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.textEditSupp->setPlainText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Rectangles");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	expressionList=node.toElement().text().split("#");
	}

nodelist = doc.elementsByTagName("Couleurs");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	couleurList=node.toElement().text().split("#");
	}

nodelist = doc.elementsByTagName("Styles");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	styleList=node.toElement().text().split("#");
	}
fichier.close();
ui.listWidgetRectangles->clear();
for (int i = 0; i < expressionList.count(); i++)
	{
	if (!expressionList.at(i).isEmpty() && !couleurList.at(i).isEmpty() && !styleList.at(i).isEmpty())
		{
		QListWidgetItem *nouveauItem = new QListWidgetItem(ui.listWidgetRectangles);
		nouveauItem->setText(expressionList.at(i));
		nouveauItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		}
	else
		{
		expressionList.removeAt(i);
		couleurList.removeAt(i);
		styleList.removeAt(i);
		}
	}
if (ui.listWidgetRectangles->count()>0)
	{
	ui.listWidgetRectangles->setCurrentItem(ui.listWidgetRectangles->item(0));
    ui.listWidgetRectangles->currentItem()->setSelected(true);
	}
actualiserItemInfo();
AfficherCode();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Histogramme : ")+nomFichier);
}

void HistoDialog::ChargerHistogramme()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Histogramme : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de charger un autre graphe?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverHistogramme();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QString nouveauFichier = QFileDialog::getOpenFileName(this,QString::fromUtf8("Ouvrir Fichier"),dernierRepertoire,"Histogramme (*.pag)");
if (nouveauFichier.isEmpty()) return;
Ouvrir(nouveauFichier);
}
//*********************************************

//******************************
void HistoDialog::NettoyerCode()
{
ui.textEditCode->clear();
estModifie=true;
setWindowTitle(QString::fromUtf8("Assistant Histogramme ")+QString::fromUtf8("[modifié]")+" : "+nomFichier);
}

//*******************************
void HistoDialog::Aide()
{
AideDialog *aideDialog = new AideDialog(this,"histogramme");
aideDialog->exec();
}
//*****************************
void HistoDialog::ExporterPdf()
{
this->disconnect(SIGNAL(asy_pdf_fini()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(ExporterPdfSuite()));
AsyPdfTemp();
}

void HistoDialog::ExporterPdfSuite()
{
if (!erreurProcessus)
	{
	QString pdfFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Pdf (*.pdf)"));
	if (!pdfFichier.isEmpty())
		{
		QFileInfo fi(pdfFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+pdfTempFile,pdfFichier);
		}
	}
this->disconnect(SIGNAL(asy_pdf_fini()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(VoirTemp()));
}

void HistoDialog::CopierFichier(QString origine,QString destination)
{
if (destination.isEmpty() || origine.isEmpty()) return;
QFileInfo fi_or(origine);
if (!fi_or.exists()) return;
QFile fichier_or(origine);
QFileInfo fi_dest(destination);
if (fi_dest.exists())
	{
	QFile fichier_dest(destination);
	fichier_dest.remove();
	fichier_or.copy(destination);
	}
else
	{
	fichier_or.copy(destination);
	}
}

void HistoDialog::AsyPdfTemp()
{
AfficherCode();
if (ui.textEditCode->toPlainText().contains(QRegularExpression("Erreur|erreur")))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Erreurs dans le code généré.\nLa compilation n'est pas lancée."));
	return;
	}

QFile fichier;
fichier.setFileName(tempDir+"/"+asyTempFile);
fichier.open(QIODevice::WriteOnly);
QTextCodec *codec = QTextCodec::codecForName("UTF-8");
fichier.write(codec->fromUnicode(GenererCode()));
fichier.close();

erreurProcessus=false;
asy_pdf_proc = new CmdLauncher( this, tempDir, extraPath );
connect(asy_pdf_proc, SIGNAL(jobdone()),this, SIGNAL(asy_pdf_fini()));
connect(asy_pdf_proc, SIGNAL(joberror()),this, SLOT(ErreurCompilation()));
asy_pdf_proc->launchCommand(asyCommande+" -f pdf -noView "+asyTempFile);
}

void HistoDialog::ErreurCompilation()
{
erreurProcessus=true;
}




void HistoDialog::VoirTemp()
{
if (erreurProcessus) return;
QString fileImage=tempDir+"/"+pdfTempFile;
QFileInfo fi(fileImage);
if (fi.exists()) 
	{
	if (pdfviewerWindow)
	  {
	  pdfviewerWindow->openFile(fileImage,lecteurpdfCommande);
	  }
	else
	  {
	  pdfviewerWindow=new PdfViewer(fileImage,lecteurpdfCommande,this);
	  }  
	  pdfviewerWindow->raise();
	  pdfviewerWindow->show();
	}
}
