/***************************************************************************
 *   copyright       : (C) 2012-2021 by Pascal Brachet                     *
 *   https://www.xm1math.net/pdfadd/                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   @license GPL-2.0+ <https://spdx.org/licenses/GPL-2.0+.html>           *
 ***************************************************************************/

#include "logdialog.h"

#include <QTableWidget>
#include <QTableWidgetItem>
#include <QFontMetrics>
#include <QFileInfo>
#include <QTextStream>
#include <QFile>
#include <QDir>
#include <QUrl>
#include <QRegularExpression>

LogDialog::LogDialog(QWidget *parent)
    :QDialog( parent)
{
ui.setupUi(this);

ui.OutputTableWidget->setColumnCount(5);
ui.OutputTableWidget->setRowCount(1);
QTableWidgetItem *HeaderItem = new QTableWidgetItem(" ");
HeaderItem->setTextAlignment(Qt::AlignLeft);
ui.OutputTableWidget->setHorizontalHeaderItem(0,HeaderItem);
HeaderItem = new QTableWidgetItem("File");
HeaderItem->setTextAlignment(Qt::AlignLeft);
ui.OutputTableWidget->setHorizontalHeaderItem(1,HeaderItem);
HeaderItem = new QTableWidgetItem("Type");
HeaderItem->setTextAlignment(Qt::AlignLeft);
ui.OutputTableWidget->setHorizontalHeaderItem(2,HeaderItem);
HeaderItem = new QTableWidgetItem("Line");
HeaderItem->setTextAlignment(Qt::AlignLeft);
ui.OutputTableWidget->setHorizontalHeaderItem(3,HeaderItem);
HeaderItem = new QTableWidgetItem("Message");
HeaderItem->setTextAlignment(Qt::AlignLeft);
ui.OutputTableWidget->setHorizontalHeaderItem(4,HeaderItem);

//ui.OutputTableWidget->setWordWrap(true);
ui.OutputTableWidget->setSelectionMode (QAbstractItemView::SingleSelection);
QFontMetrics fm(qApp->font());
#if QT_VERSION < QT_VERSION_CHECK(5, 11, 0)
ui.OutputTableWidget->setColumnWidth(0,fm.width("> "));
ui.OutputTableWidget->setColumnWidth(1,10*fm.width("w"));
ui.OutputTableWidget->setColumnWidth(2,fm.width("WarningWW"));
ui.OutputTableWidget->setColumnWidth(3,fm.width("Line WWWWWWWW"));
ui.OutputTableWidget->setColumnWidth(4,20*fm.width("w"));
#else
ui.OutputTableWidget->setColumnWidth(0,fm.horizontalAdvance("> "));
ui.OutputTableWidget->setColumnWidth(1,10*fm.horizontalAdvance("w"));
ui.OutputTableWidget->setColumnWidth(2,fm.horizontalAdvance("WarningWW"));
ui.OutputTableWidget->setColumnWidth(3,fm.horizontalAdvance("Line WWWWWWWW"));
ui.OutputTableWidget->setColumnWidth(4,20*fm.horizontalAdvance("w"));
#endif
connect(ui.OutputTableWidget, SIGNAL(itemClicked ( QTableWidgetItem*)), this, SLOT(ClickedOnLogLine(QTableWidgetItem*)));
ui.OutputTableWidget->setMinimumHeight(5*(fm.lineSpacing()+4));


ui.OutputTextEdit->setFocusPolicy(Qt::ClickFocus);
ui.OutputTextEdit->setMinimumHeight(3*(fm.lineSpacing()+4));
ui.OutputTextEdit->setReadOnly(true);

tempDir=QDir::tempPath();
prefixFile=QDir::homePath();
prefixFile="pdfadd_temp_"+prefixFile.section('/',-1);
prefixFile=QString(QUrl::toPercentEncoding(prefixFile));
prefixFile.remove("%");
logTempFile=tempDir+"/"+prefixFile+".log";

QString line;
QFileInfo fic(logTempFile);
if (fic.exists() && fic.isReadable() )
	{
	ui.OutputTextEdit->insertLine(QString::fromUtf8("FICHIER LOG :"));
	QFile f(logTempFile);
	if ( f.open(QIODevice::ReadOnly) )
		{
		QTextStream t( &f );
//		OutputTextEdit->setPlainText( t.readAll() );
		while ( !t.atEnd() )
			{
			line=t.readLine();
			line=line.simplified();
			if (!line.isEmpty()) ui.OutputTextEdit->append(line);
			}
		}
		f.close();
	LatexError();
	}
else {ui.OutputTextEdit->insertLine(QString::fromUtf8("FICHIER LOG NON TROUVE !!"));}

}

LogDialog::~LogDialog(){
}

void LogDialog::ClickedOnLogLine(QTableWidgetItem *item)
{
QString content=item->text();
int Start, End;
bool ok;
QString s;
QString line="";
//// l. ///
s = content;
Start=End=0;
Start=s.indexOf(QRegularExpression("l.[0-9]"), End);
if (Start!=-1)
	{
	Start=Start+2;
	s=s.mid(Start,s.length());
	End=s.indexOf(QRegularExpression("[ a-zA-Z.\\-]"),0);
	if (End!=-1)
	line=s.mid(0,End);
	else
	line=s.mid(0,s.length());
	};
//// line ///
s = content;
Start=End=0;
Start=s.indexOf(QRegularExpression("line [0-9]"), End);
if (Start!=-1)
	{
	Start=Start+5;
	s=s.mid(Start,s.length());
	End=s.indexOf(QRegularExpression("[ a-zA-Z.\\-]"),0);
	if (End!=-1)
	line=s.mid(0,End);
	else
	line=s.mid(0,s.length());
	};
//// lines ///
s = content;
Start=End=0;
Start=s.indexOf(QRegularExpression("lines [0-9]"), End);
if (Start!=-1)
	{
	Start=Start+6;
	s=s.mid(Start,s.length());
	End=s.indexOf(QRegularExpression("[ a-zA-Z.\\-]"),0);
	if (End!=-1)
	line=s.mid(0,End);
	else
	line=s.mid(0,s.length());
	};
int l=line.toInt(&ok,10)-1;
QString ll=item->data(Qt::UserRole).toString();
int logline=ll.toInt(&ok,10)-1;
ui.OutputTextEdit->setCursorPosition(logline , 0);
}

void LogDialog::LatexError()
{
errorFileList.clear();
errorTypeList.clear();
errorLineList.clear();
errorMessageList.clear();
errorLogList.clear();
onlyErrorList.clear();

QString mot,suivant,lettre,expression,warning,latexerror,badbox;
QStringList pile,filestack;
filestack.clear();
pile.clear();

int j;

int ligne=0;
int par=1;
int errorpar=1;

QRegularExpression rxWarning1("Warning: (.*) on.*line *(\\d+)");
QRegularExpression rxWarning2("Warning: (.*)");
QRegularExpression rxLatexError("(! )*(LaTeX Error:)* *(.*)\\.l\\.(\\d+) *(.*)");
QRegularExpression rxLineError("l\\.(\\d+)");
QRegularExpression rxBadBox("at (line|lines) ([0-9]+)");


QTextBlock tb = ui.OutputTextEdit->document()->begin();
while (tb.isValid())
 	{
	errorpar=par;
	expression=tb.text();
	j=0;
	pile.clear();
	while (j<expression.length())
		{
		lettre=expression.mid(j,1);
		if (lettre=="(" || lettre==")") 
			{
			pile.prepend(lettre);
			j+=1;
			}
		else
			{
			mot="";
			while (j<expression.length() && (expression.mid(j,1)!="(" && expression.mid(j,1)!=")")) 
				{
				mot+=expression.mid(j,1);
				j+=1;
				}
			pile.prepend(mot);
			}
		}
	while (pile.count()>0)
		{
		mot=pile.last();
		pile.removeLast();
		if (mot=="(" && pile.count()>0)
			{
			suivant=pile.last();
			pile.removeLast();
			filestack.append(suivant.remove("./"));
			}
		else if (mot==")" && filestack.count()>0) filestack.removeLast();
		}
	if (expression.contains("Warning"))
		{
		warning=expression.trimmed();
		while (tb.isValid() && !expression.contains(QRegularExpression("\\.$")))
			{
			par++;
			tb=tb.next();
			if (tb.isValid())
				{
				expression=tb.text();
				warning+=expression.trimmed();
				}
			}
        QRegularExpressionMatch rxWarning1Match=rxWarning1.match(warning);
        QRegularExpressionMatch rxWarning2Match=rxWarning2.match(warning);
        if (rxWarning1Match.hasMatch())
			{
			if (!filestack.isEmpty()) errorFileList.append(filestack.last());
			else errorFileList.append("");
			errorTypeList.append("Warning");
			errorLineList.append(rxWarning1Match.captured(2));
			errorMessageList.append(rxWarning1Match.captured(1).replace("*",""));
			errorLogList.append(QString::number(errorpar));
			}
        else if ( rxWarning2Match.hasMatch())
			{
			if (!filestack.isEmpty()) errorFileList.append(filestack.last());
			else errorFileList.append("");
			errorTypeList.append("Warning");
			errorLineList.append("1");
			errorMessageList.append(rxWarning2Match.captured(1).replace("*",""));
			errorLogList.append(QString::number(errorpar));
			}
		else
			{
			if (!filestack.isEmpty()) errorFileList.append(filestack.last());
			else errorFileList.append("");
			errorTypeList.append("Warning");
			errorLineList.append("1");
			errorMessageList.append(warning.replace("*",""));
			errorLogList.append(QString::number(errorpar));
 			}
		errorpar=par;
		}
	else if (expression.contains(QRegularExpression("^! (.*)")))
		{
		latexerror=expression.trimmed();
		while (tb.isValid() && !expression.contains(rxLineError))
			{
			par++;
			tb=tb.next();
			if (tb.isValid())
				{
				expression=tb.text();
				latexerror+=expression.trimmed();
				}
			}
        QRegularExpressionMatch rxLineErrorMatch=rxLineError.match(latexerror);
		if ( rxLineErrorMatch.hasMatch() )
			{
			if (!filestack.isEmpty()) errorFileList.append(filestack.last());
			else errorFileList.append("");
			errorTypeList.append("Error");
			errorLineList.append(rxLineErrorMatch.captured(1));
			errorMessageList.append(latexerror.remove(rxLineError).replace("*",""));
			errorLogList.append(QString::number(errorpar));
			}
		else
			{
			if (!filestack.isEmpty()) errorFileList.append(filestack.last());
			else errorFileList.append("");
			errorTypeList.append("Error");
			errorLineList.append("1");
			errorMessageList.append(latexerror.replace("*",""));
			errorLogList.append(QString::number(errorpar));
			}
		errorpar=par;
		}
	else if (expression.contains(QRegularExpression("^(Over|Under)(full \\\\[hv]box .*)")))
		{
		badbox=expression.trimmed();
       QRegularExpressionMatch rxBadBoxMatch=rxBadBox.match(badbox);
		if ( rxBadBoxMatch.hasMatch() )
			{
			if (!filestack.isEmpty()) errorFileList.append(filestack.last());
			else errorFileList.append("");
			errorTypeList.append("Badbox");
			errorLineList.append(rxBadBoxMatch.captured(2));
			errorMessageList.append(badbox.replace("*",""));
			errorLogList.append(QString::number(errorpar));
			}
		else
			{
			if (!filestack.isEmpty()) errorFileList.append(filestack.last());
			else errorFileList.append("");
			errorTypeList.append("Badbox");
			errorLineList.append("1");
			errorMessageList.append(badbox.replace("*",""));
			errorLogList.append(QString::number(errorpar));
			}
		errorpar=par;
		}
	if (tb.isValid()) 
		{
		par++;
		tb = tb.next();
		}
	}
DisplayLatexError();
}

void LogDialog::DisplayLatexError()
{
ui.OutputTableWidget->clearContents();
QFontMetrics fm(qApp->font());
int rowheight=fm.lineSpacing()+4;
int maxwidth=0;
int row=0;
if (errorFileList.count()>0)
	{
	ui.OutputTableWidget->setRowCount(errorFileList.count());
	for ( int i = 0; i <errorFileList.count(); i++ )
		{
		if (errorTypeList.at(i)=="Error")
			{
			QTableWidgetItem *ItemFile = new QTableWidgetItem(errorFileList.at(i));
			ItemFile->setData(Qt::UserRole,errorLogList.at(i));
			ItemFile->setForeground(QBrush(QColor(Qt::red)));
			ItemFile->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
			ui.OutputTableWidget->setItem(row,1, ItemFile);
			QTableWidgetItem *ItemType = new QTableWidgetItem(errorTypeList.at(i));
			ItemType->setData(Qt::UserRole,errorLogList.at(i));
			ItemType->setForeground(QBrush(QColor(Qt::red)));
			ItemType->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
			ui.OutputTableWidget->setItem(row,2, ItemType);
			QTableWidgetItem *ItemLine = new QTableWidgetItem("line "+errorLineList.at(i));
			ItemLine->setData(Qt::UserRole,errorLogList.at(i));
			ItemLine->setForeground(QBrush(QColor(Qt::red)));
			ItemLine->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
			ItemLine->setToolTip(QString::fromUtf8("Cliquer ici pour atteindre la ligne correspondante dans le fichier log"));
			ui.OutputTableWidget->setItem(row,3, ItemLine);
			QTableWidgetItem *ItemMessage = new QTableWidgetItem(errorMessageList.at(i));
            #if QT_VERSION < QT_VERSION_CHECK(5, 11, 0)
            if (fm.width(errorMessageList.at(i))>maxwidth) maxwidth=fm.width(errorMessageList.at(i));
            #else
			if (fm.horizontalAdvance(errorMessageList.at(i))>maxwidth) maxwidth=fm.horizontalAdvance(errorMessageList.at(i));
            #endif
			ItemMessage->setData(Qt::UserRole,errorLogList.at(i));
			ItemMessage->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
			ItemMessage->setForeground(QBrush(QColor(Qt::red)));
			ui.OutputTableWidget->setItem(row,4, ItemMessage);

			onlyErrorList.append(i);
			row++;
			}
		}
	for ( int i = 0; i <errorFileList.count(); i++ )
		{
		if (errorTypeList.at(i)!="Error")
			{
			QTableWidgetItem *ItemFile = new QTableWidgetItem(errorFileList.at(i));
			ItemFile->setData(Qt::UserRole,errorLogList.at(i));
			ItemFile->setForeground(QBrush(QColor(Qt::blue)));
			ItemFile->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
			ui.OutputTableWidget->setItem(row,1, ItemFile);
			QTableWidgetItem *ItemType = new QTableWidgetItem(errorTypeList.at(i));
			ItemType->setData(Qt::UserRole,errorLogList.at(i));
			ItemType->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
			ItemType->setForeground(QBrush(QColor(Qt::blue)));
			ui.OutputTableWidget->setItem(row,2, ItemType);
			QTableWidgetItem *ItemLine = new QTableWidgetItem("line "+errorLineList.at(i));
			ItemLine->setData(Qt::UserRole,errorLogList.at(i));
			ItemLine->setForeground(QBrush(QColor(Qt::blue)));
			ItemLine->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
			ItemLine->setToolTip(QString::fromUtf8("Cliquer ici pour atteindre la ligne correspondante dans le fichier log"));
			ui.OutputTableWidget->setItem(row,3, ItemLine);
			QTableWidgetItem *ItemMessage = new QTableWidgetItem(errorMessageList.at(i));
            #if QT_VERSION < QT_VERSION_CHECK(5, 11, 0)
			if (fm.width(errorMessageList.at(i))>maxwidth) maxwidth=fm.width(errorMessageList.at(i));
            #else
            if (fm.horizontalAdvance(errorMessageList.at(i))>maxwidth) maxwidth=fm.horizontalAdvance(errorMessageList.at(i));
            #endif
			ItemMessage->setData(Qt::UserRole,errorLogList.at(i));
			ItemMessage->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
			ItemMessage->setForeground(QBrush(QColor(Qt::blue)));
			ui.OutputTableWidget->setItem(row,4, ItemMessage);
			row++;
			}
		}
	ui.OutputTableWidget->setColumnWidth(4,maxwidth);
	for ( int i = 0; i<ui.OutputTableWidget->rowCount(); ++i )
		{
		ui.OutputTableWidget->setRowHeight(i,rowheight);
		}
	}
else
	{
	ui.OutputTableWidget->setRowCount(1);
	ui.OutputTableWidget->setRowHeight(0,rowheight);
	ui.OutputTableWidget->clearContents();
	}
ui.OutputTableWidget->show();
ui.OutputTextEdit->setCursorPosition(0 , 0);
}

