/***************************************************************************
 *   copyright       : (C) 2012-2021 by Pascal Brachet                     *
 *   https://www.xm1math.net/pdfadd/                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   @license GPL-2.0+ <https://spdx.org/licenses/GPL-2.0+.html>           *
 ***************************************************************************/

#include "suitesdialog.h"
#include "aidedialog.h"
#include "logdialog.h"

#include <QDebug>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QDomNodeList>
#include <QDomProcessingInstruction>
#include <QFile>
#include <QDir>
#include <QMessageBox>
#include <QFileDialog>
#include <QFileInfo>
#include <QUrl>
#include <QFontDatabase>
#include <QRegularExpression>
#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
    #include <QtCore/QTextCodec>
#else
    #include <QtCore5Compat/QTextCodec>
#endif


#include <math.h>

SuitesDialog::SuitesDialog(QWidget *parent)
    :QDialog( parent)
{

ui.setupUi(this);
#if defined(Q_OS_MAC)
setModal(true);
#endif

connect(ui.barreOutils->actionQuitter, SIGNAL(triggered()), this, SLOT(accept()));
connect(ui.barreOutils->actionGenerer, SIGNAL(triggered()), this, SLOT(AfficherCode()));
connect(ui.barreOutils->actionNouveau, SIGNAL(triggered()), this, SLOT(NouvelleSuite()));
connect(ui.barreOutils->actionOuvrir, SIGNAL(triggered()), this, SLOT(ChargerSuite()));
connect(ui.barreOutils->actionSauver, SIGNAL(triggered()), this, SLOT(SauverSuite()));
connect(ui.barreOutils->actionAide, SIGNAL(triggered()), this, SLOT(Aide()));
connect(ui.barreOutils->actionExporterPdf, SIGNAL(triggered()), this, SLOT(ExporterPdf()));
connect(ui.barreOutils->actionApercu, SIGNAL(triggered()), this, SLOT(AsyPdfTemp()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(VoirTemp()));

connect(ui.lineEditFonction, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditVarmin, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditVarmax, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.comboBoxCouleur, SIGNAL(currentTextChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.comboBoxStyle, SIGNAL(currentTextChanged(const QString &)),this,SLOT(NettoyerCode()));

connect(ui.lineEditU0, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.comboBoxCouleurSuite, SIGNAL(currentTextChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.comboBoxStyleSuite, SIGNAL(currentTextChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.spinBoxNbtermes, SIGNAL(valueChanged(int)),this, SLOT(NettoyerCode()));

connect(ui.checkBoxAxes, SIGNAL(toggled(bool)),this, SLOT(ActiverAxes(bool)));
connect(ui.checkBoxGrille, SIGNAL(toggled(bool)),this, SLOT(ActiverGrille(bool)));

connect(ui.lineEditUniteX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditXmin, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditXmax, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGrilleX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGradX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));

connect(ui.lineEditUniteY, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditYmin, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditYmax, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGrilleY, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGradY, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));

connect(ui.textEditSupp, SIGNAL(textChanged()),this,SLOT(NettoyerCode()));

connect(ui.checkBoxRepere, SIGNAL(toggled(bool)),this, SLOT(NettoyerCode()));
connect(ui.checkBoxLabels, SIGNAL(toggled(bool)),this, SLOT(NettoyerCode()));



ui.comboBoxCouleur->addItems(QString("couleur1,couleur2,couleur3").split(","));
ui.comboBoxCouleurSuite->addItems(QString("couleur1,couleur2,couleur3").split(","));

tempDir=QDir::tempPath();
prefixFile=QDir::homePath();
prefixFile="pdfadd_temp_"+prefixFile.section('/',-1);
prefixFile=QString(QUrl::toPercentEncoding(prefixFile));
prefixFile.remove("%");
asyTempFile=prefixFile+".asy";
pdfTempFile=prefixFile+".pdf";

highlighterSupp = new Highlighter(ui.textEditSupp->document());

QFont efont;
QFontDatabase fdb;
QStringList xf = fdb.families();
QString deft;
if (xf.contains("Liberation Mono",Qt::CaseInsensitive)) deft="Liberation Mono";
else if (xf.contains("DejaVu Sans Mono",Qt::CaseInsensitive)) deft="DejaVu Sans Mono";
#if defined(Q_OS_MAC)
else if (xf.contains("Courier",Qt::CaseInsensitive)) deft="Courier";
#endif
#if defined(Q_OS_WIN32)
else if (xf.contains("Courier New",Qt::CaseInsensitive)) deft="Courier New";
#endif
else deft=qApp->font().family();  
efont.setFamily(deft);
efont.setPointSize(qApp->font().pointSize());
ui.textEditCode->setFont(efont);
ui.textEditSupp->setFont(efont);
ui.textEditCode->setWordWrapMode(QTextOption::NoWrap);
ui.textEditSupp->setWordWrapMode(QTextOption::NoWrap);
ui.lineEditFonction->setFont(efont);
ui.lineEditU0->setFont(efont);
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Suites récurrentes : ")+nomFichier);
show();
}

SuitesDialog::~SuitesDialog(){
// if (asy_pdf_proc && asy_pdf_proc->state()==QProcess::Running)
// 	{
// 	asy_pdf_proc->disconnect();
// 	asy_pdf_proc->kill();
// 	delete asy_pdf_proc ;
// 	}
}

void SuitesDialog::closeEvent(QCloseEvent *e)
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Suites récurrentes : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverSuite();
		break;
		case 1:
		break;
		case 2:
		default:
		e->ignore();
		return;
		break;
		}
	}
e->accept();
}

void SuitesDialog::accept()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Suites récurrentes : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverSuite();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::accept();
}

void SuitesDialog::reject()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Suites récurrentes : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverSuite();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::reject();
}

void SuitesDialog::Init()
{
ui.textEditCode->clear();
ui.checkBoxAxes->setChecked(true);
ui.checkBoxLabels->setChecked(false);
ui.checkBoxRepere->setChecked(true);
ui.checkBoxGrille->setChecked(false);
ActiverGrille(false);
ui.lineEditUniteX->setText("1");
ui.lineEditXmin->setText("0");
ui.lineEditXmax->setText("1");
ui.lineEditGradX->setText("1");
ui.lineEditGrilleX->setText("1");
ui.lineEditUniteY->setText("1");
ui.lineEditYmin->setText("0");
ui.lineEditYmax->setText("1");
ui.lineEditGradY->setText("1");
ui.lineEditGrilleY->setText("1");
ui.lineEditFonction->setText("");
ui.lineEditVarmin->setText("0");
ui.lineEditVarmax->setText("1");

ui.lineEditU0->setText("u;0");
ui.spinBoxNbtermes->setValue(3);
ui.comboBoxCouleurSuite->setCurrentIndex(1);
ui.comboBoxStyleSuite->setCurrentIndex(2);

ui.textEditSupp->clear();
}
//*********************************************
void SuitesDialog::ActiverAxes(bool etat)
{
ui.lineEditGradX->setEnabled(etat);
ui.lineEditGradY->setEnabled(etat);
ui.checkBoxLabels->setEnabled(etat);
NettoyerCode();
}
void SuitesDialog::ActiverGrille(bool etat)
{
ui.lineEditGrilleX->setEnabled(etat);
ui.lineEditGrilleY->setEnabled(etat);
NettoyerCode();
}
//******************************************************************
bool SuitesDialog::Operateur(QString mot)
{
bool result=false;
if ( (mot=="(") || (mot==")") || (mot=="-") || (mot=="+") || (mot=="*") || (mot=="/") || (mot=="^") ) result=true;
return result;
}

bool SuitesDialog::Fonction(QString mot)
{
bool result=false;
if ( (mot=="cos") || (mot=="sin") || (mot=="sqrt") || (mot=="ln") || (mot=="log") || (mot=="exp") || (mot=="tan") || (mot=="abs") || (mot=="acos") || (mot=="asin") || (mot=="atan") ) result=true;
return result;
}

int SuitesDialog::Priorite(QString mot)
{
int result=-1;
if (mot=="(") result=0;
else if (mot==")") result=1;
else if (mot=="-") result=2;
else if (mot=="+") result=2;
else if (mot=="*") result=3;
else if (mot=="/") result=3;
else if (mot=="^") result=5;
else if (Fonction(mot)) result=4;
return result;
}

QString SuitesDialog::CalculOp(QString n,QString m,QString x,QString variable)
{
double calcul=0;
double p,q;
bool ok;
if (n=="x") p=variable.toDouble(&ok);
else p=n.toDouble(&ok);
if (m=="x") q=variable.toDouble(&ok);
else q=m.toDouble(&ok);
if (x=="+") calcul=q+p;
if (x=="-") calcul=q-p;
if (x=="*") calcul=q*p;
if (x=="/") 
	{
	if (p!=0) calcul=q/p;
	else existvaleur=false;
	}
if (x=="^") 
{
	if ((floor(abs(p))==(abs(p))) && (p>=0)) 
		{
		if (p==0) calcul=1;
		if (p==1) calcul=q;
		if (p>1) 
			{
			calcul=1;
			for (int i = 1; i <=p; i++) calcul=calcul*q;
			}
		}
	else 
		{
		if (q>1e-4) calcul=exp(p*log(abs(q)));
		else existvaleur=false;
		}
}
return QString::number(calcul,'g',12);
}

QString SuitesDialog::CalculAp(QString n,QString x,QString variable)
{
double calcul=0;
double p;
bool ok;
if (n=="x") p=variable.toDouble(&ok);
else p=n.toDouble(&ok);
if (x=="cos")  calcul=cos(p);
if (x=="sin")  calcul=sin(p);
if (x=="abs")  calcul=abs(p);
if (x=="sqrt") 
   {
   if (p>=0)  calcul=sqrt(p);
   else existvaleur=false;
   }
if (x=="ln") 
   {
   if (p>1e-50)  calcul=log(p);
   else existvaleur=false;
   }
if (x=="exp") 
   {
   if ((p<80) && (p>-80))  calcul=exp(p);
   else existvaleur=false;
   }
if (x=="tan") 
   {
   if (cos(p)!=0) calcul=tan(p);
   else existvaleur=false;
   }
if (x=="acos") 
   {
   if ((p<=1) && (p>=-1)) calcul=acos(p);
   else existvaleur=false;
   }
if (x=="asin") 
   {
   if ((p<=1) && (p>=-1)) calcul=asin(p);
   else existvaleur=false;
   }
if (x=="atan") calcul=atan(p);
if (x=="log") 
   {
   if (p>1e-50)  calcul=log10(p);
   else existvaleur=false;
   }
return QString::number(calcul,'g',12);
}


QString SuitesDialog::CalculerExpression(QString expression, QString variable)
{
QStringList Pp,Rp,Ep,Cp;
Pp.clear();
Rp.clear();
Ep.clear();
Cp.clear();
QString lettre,mot,n,m,z,result;
//**** PARSEUR et CREATION PILES
int i;
bool fini;
if (expression.isEmpty()) return result;
expression+="f";
i=0;
fini=false;
do
	{
	lettre=expression.mid(i,1);
	i+=1;
	if (lettre=="-")
		{
		if (Ep.isEmpty()) Ep.append("0");
		else if (Ep.last()=="(") Ep.append("0");
		}
	if (Operateur(lettre) || lettre=="x") Ep.append(lettre);
	else
		{
		mot="";
		while ((!Operateur(lettre)) && (lettre!="f")) 
			{
			mot+=lettre;
			lettre=expression.mid(i,1);
			i+=1;
			}
		if (mot=="pi") mot="3.141592653589793238";
		if (!mot.isEmpty()) Ep.append(mot);
		if (lettre=="f") 
			{
			fini=true;
			Ep.append("f");
			}
		else Ep.append(lettre);
		}
	}
while (!fini);
//******* POSTFIXE
i=0;
mot="";
Pp.append("(");
mot=Ep.at(i);
while (mot!="f") 
	{
	if ((!Operateur(mot)) && (!Fonction(mot))) Rp.append(mot);
	else
		{
		if (mot=="(") Pp.append(mot);
		else
			{
			while (Priorite(Pp.last())>=Priorite(mot)) 
				{
				Rp.append(Pp.last());
				Pp.removeLast();
				}
			if (mot==")")  Pp.removeLast();
			if (mot!=")")  Pp.append(mot);
			}
		}
	i++;
	mot=Ep.at(i);
	}
while (Pp.last()!="(" && !Pp.isEmpty())
      {
      Rp.append(Pp.last());
      Pp.removeLast();
      }
if (Rp.isEmpty()) Rp.append("0");
//******* CALCULER
existvaleur=true;
for (int ind=0;ind<Rp.count();ind++) 
	{
	mot=Rp.at(ind);
	if (Operateur(mot)) 
		{
		n=Cp.last();
		Cp.removeLast();
		m=Cp.last();
		Cp.removeLast();
		z=CalculOp(n,m,mot,variable);
		Cp.append(z);
		}
	else
	{
	if (Fonction(mot))
		{
		n=Cp.last();
		Cp.removeLast();
		z=CalculAp(n,mot,variable);
		Cp.append(z);
		}
	else 
		{
		Cp.append(mot);
		}
	}
	}
if (Cp.first()=="x") result=variable;
else result=Cp.first();
if (!existvaleur) result="Erreur";
return result;
}

QString SuitesDialog::CalculerExpressionArrondi(QString expression, QString variable)
{
QString result;
double num_result, num_resultbis;
int approx_result;
bool ok;
result=CalculerExpression(expression,variable);
if (result!="Erreur") 
	{
	num_result=result.toDouble(&ok);
	if (ok) 
		{
		num_resultbis=floor (num_result*1e5);
		approx_result=(int) num_resultbis;
		num_result=(double) (approx_result/1e5);
		result=QString::number(num_result,'g',12);
		}
	}
return result;
}

QString SuitesDialog::GenererCode()
{
QString couleur,style,couleur_suite,style_suite,s_varmin, s_varmax,label_suite,valeur_initiale;

QStringList val=ui.lineEditU0->text().split(";");
if (val.count()<1) return "Erreur";
if (val.count()==2)
	{
	label_suite=val.at(0);
	valeur_initiale=val.at(1);
	}
else 
	{
	label_suite="rien";
	valeur_initiale=val.at(0);
	}

bool ok;

QString s_unitX=CalculerExpressionArrondi(ui.lineEditUniteX->text(),"1");
double unitX;
if (s_unitX!="Erreur") unitX=s_unitX.toDouble(&ok);
else unitX=1;

QString s_unitY=CalculerExpressionArrondi(ui.lineEditUniteY->text(),"1");
double unitY;
if (s_unitY!="Erreur") unitY=s_unitY.toDouble(&ok);
else unitY=1;

QString s_Xmin=CalculerExpressionArrondi(ui.lineEditXmin->text(),"1");
double Xmin;
if (s_Xmin!="Erreur") Xmin=s_Xmin.toDouble(&ok);
else Xmin=0;

QString s_Ymin=CalculerExpressionArrondi(ui.lineEditYmin->text(),"1");
double Ymin;
if (s_Ymin!="Erreur") Ymin=s_Ymin.toDouble(&ok);
else Ymin=0;

QString s_Xmax=CalculerExpressionArrondi(ui.lineEditXmax->text(),"1");
double Xmax;
if (s_Xmax!="Erreur") Xmax=s_Xmax.toDouble(&ok);
else Xmax=1;

QString s_Ymax=CalculerExpressionArrondi(ui.lineEditYmax->text(),"1");
double Ymax;
if (s_Ymax!="Erreur") Ymax=s_Ymax.toDouble(&ok);
else Ymax=1;

QString s_gradX=CalculerExpressionArrondi(ui.lineEditGradX->text(),"1");
double gradX;
if (s_gradX!="Erreur") gradX=s_gradX.toDouble(&ok);
else gradX=1;

QString s_gradY=CalculerExpressionArrondi(ui.lineEditGradY->text(),"1");
double gradY;
if (s_gradY!="Erreur") gradY=s_gradY.toDouble(&ok);
else gradY=1;

QString s_grilleX=CalculerExpressionArrondi(ui.lineEditGrilleX->text(),"1");
double grilleX;
if (s_grilleX!="Erreur") grilleX=s_grilleX.toDouble(&ok);
else grilleX=1;
if (grilleX<=0) grilleX=1;

QString s_grilleY=CalculerExpressionArrondi(ui.lineEditGrilleY->text(),"1");
double grilleY;
if (s_grilleY!="Erreur") grilleY=s_grilleY.toDouble(&ok);
else grilleY=1;
if (grilleY<=0) grilleY=1;

double grilleXmin=Xmin/grilleX;
double grilleXmax=Xmax/grilleX;
double grilleYmin=Ymin/grilleY;
double grilleYmax=Ymax/grilleY;
double grilleUnitX=grilleX*unitX;
double grilleUnitY=grilleY*unitY;
QString s_Ox="0";
QString s_Oy="0";
if (Xmin*Xmax>0) s_Ox=s_Xmin;
if (Ymin*Ymax>0) s_Oy=s_Ymin;
// if (Xmin>0) s_Ox="\\xmin";
// if (Xmax<0) s_Ox="\\xmax";
// if (Ymin>0) s_Oy="\\ymin";
// if (Ymax<0) s_Oy="\\ymax";
QString texcode=QString::fromUtf8("// préambule asymptote\n")+preambuleAsy+"\n// code figure\n";
texcode+="import graph;\n";
texcode+="unitsize("+s_unitX+"cm,"+s_unitY+"cm);\n";
texcode+="xlimits("+s_Xmin+","+s_Xmax+");\n";
texcode+="ylimits("+s_Ymin+","+s_Ymax+");\n";

int pR1= (couleurSupp1.red()*10000/255);
double cR1=(double) pR1/10000;
int pG1= (couleurSupp1.green()*10000/255);
double cG1=(double) pG1/10000;
int pB1= (couleurSupp1.blue()*10000/255);
double cB1=(double) pB1/10000;
QString color1="rgb("+QString::number(cR1 ,'g',5)+","+QString::number(cG1 ,'g',5)+","+QString::number(cB1 ,'g',5)+")";

int pR2= (couleurSupp2.red()*10000/255);
double cR2=(double) pR2/10000;
int pG2= (couleurSupp2.green()*10000/255);
double cG2=(double) pG2/10000;
int pB2= (couleurSupp2.blue()*10000/255);
double cB2=(double) pB2/10000;
QString color2="rgb("+QString::number(cR2 ,'g',5)+","+QString::number(cG2 ,'g',5)+","+QString::number(cB2 ,'g',5)+")";

int pR3= (couleurSupp3.red()*10000/255);
double cR3=(double) pR3/10000;
int pG3= (couleurSupp3.green()*10000/255);
double cG3=(double) pG3/10000;
int pB3= (couleurSupp3.blue()*10000/255);
double cB3=(double) pB3/10000;
QString color3="rgb("+QString::number(cR3 ,'g',5)+","+QString::number(cG3 ,'g',5)+","+QString::number(cB3 ,'g',5)+")";

if (ui.comboBoxCouleurSuite->currentText()=="noir") couleur_suite="black";
else if (ui.comboBoxCouleurSuite->currentText()=="gris") couleur_suite="gray";
else if (ui.comboBoxCouleurSuite->currentText()==QString::fromUtf8("gris foncé")) couleur_suite="heavygray";
else if (ui.comboBoxCouleurSuite->currentText()=="gris clair") couleur_suite="mediumgray";
else if (ui.comboBoxCouleurSuite->currentText()=="bleu") couleur_suite="blue";
else if (ui.comboBoxCouleurSuite->currentText()=="rouge") couleur_suite="red";
else if (ui.comboBoxCouleurSuite->currentText()=="vert") couleur_suite="green";
else if (ui.comboBoxCouleurSuite->currentText()=="jaune") couleur_suite="yellow";
else if (ui.comboBoxCouleurSuite->currentText()=="cyan") couleur_suite="cyan";
else if (ui.comboBoxCouleurSuite->currentText()=="magenta") couleur_suite="magenta";
else if (ui.comboBoxCouleurSuite->currentText()=="couleur1") couleur_suite=color1;
else if (ui.comboBoxCouleurSuite->currentText()=="couleur2") couleur_suite=color2;
else if (ui.comboBoxCouleurSuite->currentText()=="couleur3") couleur_suite=color3;
else couleur_suite="black";
if (ui.comboBoxStyleSuite->currentText()=="ligne") style_suite="solid";
else if (ui.comboBoxStyleSuite->currentText()=="trait") style_suite="dashed";
else if (ui.comboBoxStyleSuite->currentText()=="point") style_suite="dotted";
else style="solid";

texcode+="real F(real x) {return "+ui.lineEditFonction->text()+";}\n";
texcode+="real s_abs="+CalculerExpression(valeur_initiale,"1")+";\n";
texcode+="real s_ord=0;\n";
texcode+="string nomsuite=\""+label_suite+"\";\n";
texcode+="for (int i = 0; i <"+QString::number( ui.spinBoxNbtermes->value())+"; ++i)\n";
texcode+="{\n";
texcode+="s_ord=F(s_abs);\n";
texcode+="draw((s_abs,0)--(s_abs,s_ord)--(0,s_ord)--(s_ord,s_ord)--(s_ord,0),linewidth(0.5pt)+white+solid);\n";
texcode+="draw((s_abs,0)--(s_abs,s_ord)--(0,s_ord)--(s_ord,s_ord)--(s_ord,0),linewidth(0.5pt)+"+couleur_suite+"+"+style_suite+");\n";
texcode+="if (i>0)\n";
texcode+="{\n";
texcode+="draw((s_abs,s_abs)--(s_abs,s_ord)--(s_ord,s_ord),linewidth(1.2pt)+white+solid,Arrow(SimpleHead,3mm));\n";
texcode+="draw((s_abs,s_abs)--(s_abs,s_ord),linewidth(1.2pt)+"+couleur_suite+"+"+style_suite+",Arrow(SimpleHead,3mm));\n";
texcode+="draw((s_abs,s_ord)--(s_ord,s_ord),linewidth(1.2pt)+"+couleur_suite+"+"+style_suite+",Arrow(SimpleHead,3mm));\n";
texcode+="}\n";
texcode+="else\n";
texcode+="{\n";
texcode+="draw((s_abs,0)--(s_abs,s_ord)--(s_ord,s_ord),linewidth(1.2pt)+white+solid,Arrow(SimpleHead,3mm));\n";
texcode+="draw((s_abs,0)--(s_abs,s_ord),linewidth(1.2pt)+"+couleur_suite+"+"+style_suite+",Arrow(SimpleHead,3mm));\n";
texcode+="draw((s_abs,s_ord)--(s_ord,s_ord),linewidth(1.2pt)+"+couleur_suite+"+"+style_suite+",Arrow(SimpleHead,3mm));\n";
texcode+="}\n";
// texcode+="draw((s_abs,s_abs)--(s_abs,s_ord),linewidth(1.2pt)+"+couleur_suite+"+"+style_suite+",Arrow(SimpleHead));\n";
// texcode+="draw((s_abs,s_ord)--(s_ord,s_ord),linewidth(1.2pt)+"+couleur_suite+"+"+style_suite+",Arrow(SimpleHead));\n";
texcode+="s_abs=s_ord;\n";
texcode+="}\n";

if (ui.comboBoxCouleur->currentText()=="noir") couleur="black";
else if (ui.comboBoxCouleur->currentText()=="gris") couleur="gray";
else if (ui.comboBoxCouleur->currentText()==QString::fromUtf8("gris foncé")) couleur="heavygray";
else if (ui.comboBoxCouleur->currentText()=="gris clair") couleur="mediumgray";
else if (ui.comboBoxCouleur->currentText()=="bleu") couleur="blue";
else if (ui.comboBoxCouleur->currentText()=="rouge") couleur="red";
else if (ui.comboBoxCouleur->currentText()=="vert") couleur="green";
else if (ui.comboBoxCouleur->currentText()=="jaune") couleur="yellow";
else if (ui.comboBoxCouleur->currentText()=="cyan") couleur="cyan";
else if (ui.comboBoxCouleur->currentText()=="magenta") couleur="magenta";
else if (ui.comboBoxCouleur->currentText()=="couleur1") couleur=color1;
else if (ui.comboBoxCouleur->currentText()=="couleur2") couleur=color2;
else if (ui.comboBoxCouleur->currentText()=="couleur3") couleur=color3;
else couleur="black";
if (ui.comboBoxStyle->currentText()=="ligne") style="solid";
else if (ui.comboBoxStyle->currentText()=="trait") style="dashed";
else if (ui.comboBoxStyle->currentText()=="point") style="dotted";
else style="solid";
s_varmin=CalculerExpression(ui.lineEditVarmin->text(),"1");
s_varmax=CalculerExpression(ui.lineEditVarmax->text(),"1");
texcode+="draw(graph(new real(real x){return x;},"+s_varmin+","+s_varmax+"),linewidth(1pt)+heavygray+solid);\n";
texcode+="draw(graph(F,"+s_varmin+","+s_varmax+",n=400),linewidth(1pt)+"+couleur+"+"+style+");\n";

if (ui.checkBoxGrille->isChecked())
  {
   texcode+="xaxis(BottomTop,xmin="+s_Xmin+",xmax="+s_Xmax+",Ticks(\"%\",extend=true,Step="+s_grilleX+"),p=linewidth(0.5pt)+gray(0.4));\n";
   texcode+="yaxis(LeftRight,ymin="+s_Ymin+",ymax="+s_Ymax+",Ticks(\"%\",extend=true,Step="+s_grilleY+"),p=linewidth(0.5pt)+gray(0.4));\n";
  }
texcode+="xlimits("+s_Xmin+","+s_Xmax+",Crop);\n";
texcode+="ylimits("+s_Ymin+","+s_Ymax+",Crop);\n";
QString ticksX="";
QString ticksY="";
QString arrow=",Arrow(2mm)";
if (ui.checkBoxRepere->isChecked() && (Xmin<=0) && (Xmax>=0) && (Ymin<=0) && (Ymax>=0)) arrow="";
if (ui.checkBoxAxes->isChecked())
  {
  if (ui.checkBoxLabels->isChecked())
    {
    if (ui.checkBoxRepere->isChecked())
	{
	ticksX=",Ticks(scale(.7)*Label(),beginlabel=true,endlabel=true,begin=true,end=true,NoZero,Step="+s_gradX+",Size=1mm)";
	ticksY=",Ticks(scale(.7)*Label(),beginlabel=true,endlabel=true,begin=true,end=true,NoZero,Step="+s_gradY+",Size=1mm)";
	}
    else
	{
	if (Ymin==0) ticksX=",Ticks(scale(.7)*Label(),beginlabel=true,endlabel=true,begin=true,end=true,Step="+s_gradX+",Size=1mm)";
	else ticksX=",Ticks(scale(.7)*Label(),beginlabel=true,endlabel=true,begin=true,end=true,NoZero,Step="+s_gradX+",Size=1mm)";
	if (Xmin==0) ticksY=",Ticks(scale(.7)*Label(),beginlabel=true,endlabel=true,begin=true,end=true,Step="+s_gradY+",Size=1mm)";	else ticksY=",Ticks(scale(.7)*Label(),beginlabel=true,endlabel=true,begin=true,end=true,NoZero,Step="+s_gradY+",Size=1mm)";
	}
    }
  else
    {
    ticksX=",Ticks(\"%\",NoZero,Step="+s_gradX+",Size=1mm)";
    ticksY=",Ticks(\"%\",NoZero,Step="+s_gradY+",Size=1mm)";
    }
  texcode+="xaxis(axis=YEquals("+s_Oy+"),xmin="+s_Xmin+",xmax="+s_Xmax+ticksX+",p=linewidth(1pt)+black"+arrow+",true);\n";
  texcode+="yaxis(axis=XEquals("+s_Ox+"),ymin="+s_Ymin+",ymax="+s_Ymax+ticksY+",p=linewidth(1pt)+black"+arrow+",true);\n";
  }
if (ui.checkBoxRepere->isChecked() && (Xmin<=0) && (Xmax>=0) && (Ymin<=0) && (Ymax>=0))
  {
  texcode+="labelx(Label(\"$O$\",NoFill), 0, SW);\n";
  texcode+="draw(Label(\"$\\vec{\\imath}$\",NoFill), (0,0)--(1,0),S,Arrow(2mm));\n";
  texcode+="draw(Label(\"$\\vec{\\jmath}$\",NoFill), (0,0)--(0,1),W,Arrow(2mm));\n";
  texcode+="dot((0,0));\n";
  }
texcode+="if (nomsuite!=\"rien\")\n";
texcode+="{\n";
texcode+="s_abs="+CalculerExpression(valeur_initiale,"1")+";\n";
texcode+="s_ord=0;\n";
texcode+="for (int i = 0; i <"+QString::number( ui.spinBoxNbtermes->value())+"; ++i)\n";
texcode+="{\n";
texcode+="if (i>0) label(\"$\"+nomsuite+\"_{\"+(string) i+\"}$\",("+s_Ox+",s_abs),4*W);\n";
texcode+="label(\"$\"+nomsuite+\"_{\"+(string) i+\"}$\",(s_abs,"+s_Oy+"),4*S);\n";
texcode+="s_ord=F(s_abs);\n";
texcode+="s_abs=s_ord;\n";
texcode+="}\n";
texcode+="label(\"$\"+nomsuite+\"_{"+QString::number(ui.spinBoxNbtermes->value())+"}$\",("+s_Ox+",s_abs),4*W);\n";
texcode+="label(\"$\"+nomsuite+\"_{"+QString::number(ui.spinBoxNbtermes->value())+"}$\",(s_abs,"+s_Oy+"),4*S);\n";
texcode+="}\n";
texcode+=QString::fromUtf8("// code supplémentaire\n")+ui.textEditSupp->toPlainText()+"\n";   
texcode+="shipout(bbox(0.1cm,0.1cm,white));\n";
return texcode;
}

void SuitesDialog::AfficherCode()
{
ui.textEditCode->clear();
QString texcode=GenererCode()+"\n";
ui.textEditCode->setPlainText(texcode);
}
//******************************************************
void SuitesDialog::NouvelleSuite()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Suites récurrentes : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant d'en créer un nouveau?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverSuite();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Suites récurrentes : ")+nomFichier);
}

void SuitesDialog::SauverSuite()
{
QDomDocument doc;
QDomElement description;
QDomText contenu;
QDomProcessingInstruction instr =  doc.createProcessingInstruction("xml","version=\"1.0\" encoding=\"UTF-8\"");
doc.appendChild(instr);
QDomElement root=doc.createElement("Suite");
doc.appendChild(root);

description=doc.createElement("UnitX");
contenu=doc.createTextNode(ui.lineEditUniteX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Xmin");
contenu=doc.createTextNode(ui.lineEditXmin->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Xmax");
contenu=doc.createTextNode(ui.lineEditXmax->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("UnitY");
contenu=doc.createTextNode(ui.lineEditUniteY->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Ymin");
contenu=doc.createTextNode(ui.lineEditYmin->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Ymax");
contenu=doc.createTextNode(ui.lineEditYmax->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GradX");
contenu=doc.createTextNode(ui.lineEditGradX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GradY");
contenu=doc.createTextNode(ui.lineEditGradY->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GrilleX");
contenu=doc.createTextNode(ui.lineEditGrilleX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GrilleY");
contenu=doc.createTextNode(ui.lineEditGrilleY->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Axes");
if (ui.checkBoxAxes->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Labels");
if (ui.checkBoxLabels->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Repere");
if (ui.checkBoxRepere->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Grille");
if (ui.checkBoxGrille->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Fonction");
contenu=doc.createTextNode(ui.lineEditFonction->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Varmin");
contenu=doc.createTextNode(ui.lineEditVarmin->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Varmax");
contenu=doc.createTextNode(ui.lineEditVarmax->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Couleur");
contenu=doc.createTextNode(ui.comboBoxCouleur->currentText());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Style");
contenu=doc.createTextNode(ui.comboBoxStyle->currentText());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("PremierTerme");
contenu=doc.createTextNode(ui.lineEditU0->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("SuiteCouleur");
contenu=doc.createTextNode(ui.comboBoxCouleurSuite->currentText());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("SuiteStyle");
contenu=doc.createTextNode(ui.comboBoxStyleSuite->currentText());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("NbTermes");
contenu=doc.createTextNode(QString::number(ui.spinBoxNbtermes->value()));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Supplement");
contenu=doc.createTextNode(ui.textEditSupp->toPlainText());
root.appendChild(description);
description.appendChild(contenu);

QString nouveauFichier;
if (nomFichier=="sanstitre" ) 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Suites (*.pag)"));
	}
else
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),nomFichier,QString::fromUtf8("Suites (*.pag)"));
	}
//nouveauFichier=nomFichier;
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::WriteOnly))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible d'enregistrer le fichier"));
	return;
	}
nomFichier=nouveauFichier;
QTextStream out (&fichier);
doc.save(out,4);
fichier.close();
QFileInfo fi(nomFichier);
dernierRepertoire=fi.absolutePath();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Suites récurrentes : ")+nomFichier);
}

void SuitesDialog::Ouvrir(QString nouveauFichier)
{
QDomDocument doc;
QDomNodeList nodelist;
QDomNode node;
QFileInfo fi(nouveauFichier);
if (!fi.exists()) return;
dernierRepertoire=fi.absolutePath();
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::ReadOnly)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	return;
	}
if (!doc.setContent(&fichier)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	fichier.close();
	return;
	}

QDomElement root = doc.documentElement();
if (root.tagName() != "Suite")
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Ce fichier ne correspond pas à une suite récurrente"));
	fichier.close();
	return;
	}
nomFichier=nouveauFichier;
Init();

nodelist = doc.elementsByTagName("UnitX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditUniteX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Xmin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditXmin->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Xmax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditXmax->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("UnitY");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditUniteY->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Ymin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditYmin->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Ymax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditYmax->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GradX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGradX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GradY");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGradY->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GrilleX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGrilleX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GrilleY");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGrilleY->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Axes");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxAxes->setChecked(true);
	else ui.checkBoxAxes->setChecked(false);
	}

nodelist = doc.elementsByTagName("Labels");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxLabels->setChecked(true);
	else ui.checkBoxLabels->setChecked(false);
	}

nodelist = doc.elementsByTagName("Repere");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxRepere->setChecked(true);
	else ui.checkBoxRepere->setChecked(false);
	}

nodelist = doc.elementsByTagName("Grille");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxGrille->setChecked(true);
	else ui.checkBoxGrille->setChecked(false);
	}

nodelist = doc.elementsByTagName("Fonction");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditFonction->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Varmin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditVarmin->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Varmax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditVarmax->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Couleur");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.comboBoxCouleur->setCurrentIndex(ui.comboBoxCouleur->findText(node.toElement().text(),Qt::MatchExactly));
	}

nodelist = doc.elementsByTagName("Style");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.comboBoxStyle->setCurrentIndex(ui.comboBoxStyle->findText(node.toElement().text(),Qt::MatchExactly));
	}

nodelist = doc.elementsByTagName("PremierTerme");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditU0->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("SuiteCouleur");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.comboBoxCouleurSuite->setCurrentIndex(ui.comboBoxCouleurSuite->findText(node.toElement().text(),Qt::MatchExactly));
	}

nodelist = doc.elementsByTagName("SuiteStyle");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.comboBoxStyleSuite->setCurrentIndex(ui.comboBoxStyleSuite->findText(node.toElement().text(),Qt::MatchExactly));
	}

nodelist = doc.elementsByTagName("NbTermes");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.spinBoxNbtermes->setValue(node.toElement().text().toInt());
	}

nodelist = doc.elementsByTagName("Supplement");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.textEditSupp->setPlainText(node.toElement().text());
	}

fichier.close();

AfficherCode();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Suites récurrentes : ")+nomFichier);
}

void SuitesDialog::ChargerSuite()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Suites récurrentes : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de charger un autre graphe?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverSuite();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QString nouveauFichier = QFileDialog::getOpenFileName(this,QString::fromUtf8("Ouvrir Fichier"),dernierRepertoire,"Suites (*.pag)");
if (nouveauFichier.isEmpty()) return;
Ouvrir(nouveauFichier);
}

//******************************
void SuitesDialog::NettoyerCode()
{
ui.textEditCode->clear();
estModifie=true;
setWindowTitle(QString::fromUtf8("Assistant Suites récurrentes ")+QString::fromUtf8("[modifié]")+" : "+nomFichier);
}

//*******************************
void SuitesDialog::Aide()
{
AideDialog *aideDialog = new AideDialog(this,"suites");
aideDialog->exec();
}
//*****************************
void SuitesDialog::ExporterPdf()
{
this->disconnect(SIGNAL(asy_pdf_fini()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(ExporterPdfSuite()));
AsyPdfTemp();
}

void SuitesDialog::ExporterPdfSuite()
{
if (!erreurProcessus)
	{
	QString pdfFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Pdf (*.pdf)"));
	if (!pdfFichier.isEmpty())
		{
		QFileInfo fi(pdfFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+pdfTempFile,pdfFichier);
		}
	}
this->disconnect(SIGNAL(asy_pdf_fini()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(VoirTemp()));
}

void SuitesDialog::CopierFichier(QString origine,QString destination)
{
if (destination.isEmpty() || origine.isEmpty()) return;
QFileInfo fi_or(origine);
if (!fi_or.exists()) return;
QFile fichier_or(origine);
QFileInfo fi_dest(destination);
if (fi_dest.exists())
	{
	QFile fichier_dest(destination);
	fichier_dest.remove();
	fichier_or.copy(destination);
	}
else
	{
	fichier_or.copy(destination);
	}
}

void SuitesDialog::AsyPdfTemp()
{
AfficherCode();
if (ui.textEditCode->toPlainText().contains(QRegularExpression("Erreur|erreur")))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Erreurs dans le code généré.\nLa compilation n'est pas lancée."));
	return;
	}

QFile fichier;
fichier.setFileName(tempDir+"/"+asyTempFile);
fichier.open(QIODevice::WriteOnly);
QTextCodec *codec = QTextCodec::codecForName("UTF-8");
fichier.write(codec->fromUnicode(GenererCode()));
fichier.close();

erreurProcessus=false;
asy_pdf_proc = new CmdLauncher( this, tempDir, extraPath );
connect(asy_pdf_proc, SIGNAL(jobdone()),this, SIGNAL(asy_pdf_fini()));
connect(asy_pdf_proc, SIGNAL(joberror()),this, SLOT(ErreurCompilation()));
asy_pdf_proc->launchCommand(asyCommande+" -f pdf -noView "+asyTempFile);
}

void SuitesDialog::ErreurCompilation()
{
erreurProcessus=true;
}




void SuitesDialog::VoirTemp()
{
if (erreurProcessus) return;
QString fileImage=tempDir+"/"+pdfTempFile;
QFileInfo fi(fileImage);
if (fi.exists()) 
	{
	if (pdfviewerWindow)
	  {
	  pdfviewerWindow->openFile(fileImage,lecteurpdfCommande);
	  }
	else
	  {
	  pdfviewerWindow=new PdfViewer(fileImage,lecteurpdfCommande,this);
	  }  
	  pdfviewerWindow->raise();
	  pdfviewerWindow->show();
	}
}
